@@ -1,5 +1,5 @@
 /* Commands for the curses interface to Xconq.
-   Copyright (C) 1986-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -9,6 +9,7 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kpublic.h"
 #include "cconq.h"
+extern void generic_do_doctrine PARAMS ((Side *side, char *str));
 extern int autofinish_start;
 extern int autofinish_count;
 extern int drawlinear;
@@ -518,16 +519,7 @@ do_distrust()
 void
 do_doctrine()
 {
-  /* (should make generic) */
-  if (cmdargstr) {
-    if (strcmp(cmdargstr, "set") == 0) {
-      notify(dside, "Set what?");
-    } else if (strncmp(cmdargstr, "set ", 4) == 0) {
-      net_set_doctrine(dside, cmdargstr + 4);
-    }
-    return;
-  }
-  notify_doctrine(dside, (cmdargstr ? cmdargstr : ""));
+    generic_do_doctrine(dside, cmdargstr);
 }
 
 void
