@@ -13,6 +13,7 @@ any later version.  See the file COPYING.  */
 #include "misc.h"
 #include "lisp.h"
 #include "imf.h"
+extern Image *largest_image PARAMS ((ImageFamily *imf));
 
 extern void syntax_error PARAMS ((Obj *x, char *msg));
 
@@ -38,6 +39,8 @@ enum {
 };
 
 static ImageFamily *new_imf PARAMS ((char *name));
+static Image *add_shrunken_image PARAMS ((ImageFamily *imf));
+static Image *add_magnified_image PARAMS ((ImageFamily *imf));
 static int bitmaps_match PARAMS ((int w, int h, Obj *lispdata, char *rawdata));
 static int color_matches_mono PARAMS ((Image *img));
 static void write_pixmap PARAMS ((FILE *fp, int w, int h, int aw, int ah,
@@ -66,6 +69,12 @@ int numpalettes = 0;
 
 ImageFile *image_files;
 
+ImageFamily *(*imf_load_hook) PARAMS ((ImageFamily *imf));
+
+ImageFamily *(*imf_interp_hook) PARAMS ((ImageFamily *imf, Image *img, int force));
+
+short write_synthetic_also;
+
 /* Create and return an image family. */
 
 static ImageFamily *
@@ -89,9 +98,12 @@ ImageFamily *imf;
 
     imf2 = new_imf(imf->name);
     memcpy(imf2, imf, sizeof(ImageFamily));
+    /* Clear the hook, we expect that the caller of this routine will
+       supply any new hook that might be necessary. */
+    imf2->hook = NULL;
     imf2->images = NULL;
     /* Clone the images. */
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
 	img2 = get_img(imf2, img->w, img->h);
 	truenext = img2->next;
 	memcpy(img2, img, sizeof(Image));
@@ -102,6 +114,7 @@ ImageFamily *imf;
 	img2->next = truenext;
 	/* Note that pointers to raw image data and suchlike can be
 	   left as-is, since they should be shared by image clones. */
+	/* (should copy anyway, for safety?) */
     }
     return imf2;
 }
@@ -148,11 +161,11 @@ char *name;
     ImageFamily *imf = NULL;
     
     if (name == NULL) {
-	init_warning("can't get an unnamed imf");
+	init_warning("can't get an unnamed image family");
 	return NULL;
     }
     if (!valid_imf_name(name)) {
-	init_warning("\"%s\" is not a valid imf name", name);
+	init_warning("\"%s\" is not a valid image family name", name);
 	return NULL;
     }
     if (images == NULL) {
@@ -179,7 +192,7 @@ char *name;
     ImageFile *imfile;
     
     if (name == NULL)
-      run_error("can't get an unnamed imfile");
+      run_error("can't get an unnamed image file");
     for (imfile = image_files; imfile != NULL; imfile = imfile->next) {
 	if (strcmp(name, imfile->name) == 0)
 	  return imfile;
@@ -217,7 +230,7 @@ void (*callback) PARAMS ((ImageFamily *imf, int loadnow));
 	} else
 	  break;
 	if (strcmp(buf1, ".") == 0
-		 && strcmp(buf2, ".") == 0)
+	    && strcmp(buf2, ".") == 0)
 	  done = TRUE;
 	else if (first) {
 	    if (strcmp(buf1, "ImageFamilyName") == 0
@@ -326,15 +339,13 @@ int w, h;
 {
     Image *img, *nimg, *previmg;
 
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
 	if (w == img->w && h == img->h)
 	  return img;
     }
     /* Not found; create a new image and add it to the family. */
     nimg = (Image *) xmalloc(sizeof(Image));
     nimg->w = w;  nimg->h = h;
-    /* Default min/max limits to actual size. */
-    nimg->minw = nimg->maxw = w;  nimg->minh = nimg->maxh = h;
     nimg->embedx = nimg->embedy = -1;
     nimg->embedw = nimg->embedh = -1;
     nimg->monodata = nimg->colrdata = nimg->maskdata = lispnil;
@@ -343,9 +354,9 @@ int w, h;
     nimg->notes = lispnil;
     nimg->bboxw = w;  nimg->bboxh = h;
     /* Rely on zeroing of xmalloc blocks to avoid clearing other fields. */
-    /* Link in order by size. */
+    /* Link in order by size, smallest first. */
     previmg = NULL;
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
 	if ((nimg->w < img->w) || (nimg->w == img->w && nimg->h < img->h))
 	  break;
 	previmg = img;
@@ -368,7 +379,7 @@ int w, h;
 {
     Image *img;
 	
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
 	if (w == img->w && h == img->h)
 	  return img;
     }
@@ -388,7 +399,7 @@ Obj *form;
 	}
 	return imf;
     } else {
-	run_warning("imf name must be a string");
+	run_warning("image family name must be a string");
     }
     return NULL;
 }
@@ -436,9 +447,6 @@ Obj *size, *parts;
     img = get_img(imf, w, h);
     if (img == NULL)
       run_error("no image?");
-    /* Kind of a hack. */
-    img->minw = img->w / 4;  img->minh = img->h / 4;
-    img->maxw = img->w * 4;  img->maxh = img->h * 4;
     if (img->w == 1 && img->h == 1) {
 	/* A color is more like a tile than an icon. */
 	img->istile = TRUE;
@@ -539,7 +547,7 @@ ImageFamily *imf;
 
     if (imf == NULL)
       return;
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
 	compute_image_bbox(img);
     }
 }
@@ -576,7 +584,7 @@ Image *img;
 	    byte = hextoi(data[j]) * 16 + hextoi(data[j+1]);
 	    j += 2;
 	} else if (img->rawmaskdata != NULL) {
-	    byte = img->rawmaskdata[i];
+	    byte = img->rawmaskdata[i] & 0xff;
 	} else {
 	    byte = 0xff;
 	}
@@ -586,9 +594,9 @@ Image *img;
 	    x1 = x2 = -1;
 	    k = 0;
 	    while (byte != 0) {
-		if (byte % 2 != 0 && x2 < 0)
+		if ((byte & 0x1) != 0 && x2 < 0)
 		  x2 = x + 7 - k;
-		byte /= 2;
+		byte >>= 1;
 		if (byte == 0 && x1 < 0)
 		  x1 = x + 7 - k;
 		++k;
@@ -609,6 +617,244 @@ Image *img;
     }
 }
 
+extern int image_pixel_at PARAMS ((Image *img, int imtype, int x, int y));
+
+extern void set_image_pixel_at PARAMS ((Image *img, int imtype, int x, int y, int val));
+
+int
+image_pixel_at(img, imtype, x, y)
+Image *img;
+int imtype, x, y;
+{
+    int rowbytes, psize, i, byte, rslt;
+    char *rawdata = NULL;
+
+    if (imtype == K_MONO_) {
+	rawdata = img->rawmonodata;
+	psize = 1;
+    } else if (imtype == K_MASK_) {
+	rawdata = img->rawmaskdata;
+	psize = 1;
+    } else {
+	rawdata = img->rawcolrdata;
+	psize = img->pixelsize;
+    }
+    if (rawdata != NULL) {
+	rowbytes = computed_rowbytes(img->w, psize);
+	i = y * rowbytes + ((x * psize) >> 3);
+	byte = rawdata[i];
+	rslt = (byte >> ((8 - psize) - ((x * psize) & 0x7))) & ((1 << psize) - 1);
+	return rslt;
+    }
+}
+
+void
+set_image_pixel_at(img, imtype, x, y, val)
+Image *img;
+int imtype, x, y, val;
+{
+    int rowbytes, psize, i, byte;
+    char *rawdata = NULL;
+
+    if (imtype == K_MONO_) {
+	rawdata = img->rawmonodata;
+	psize = 1;
+    } else if (imtype == K_MASK_) {
+	rawdata = img->rawmaskdata;
+	psize = 1;
+    } else {
+	rawdata = img->rawcolrdata;
+	psize = img->pixelsize;
+    }
+    if (rawdata != NULL) {
+	rowbytes = computed_rowbytes(img->w, psize);
+	i = y * rowbytes + ((x * psize) >> 3);
+	byte = rawdata[i];
+	byte |= val << ((8 - psize) - ((x * psize) & 0x7));
+	rawdata[i] = byte;
+    }
+}
+
+/* If an image family includes no small images (16x16 or less), then
+   add an image that is half the size of the smallest image in the
+   family. */
+
+static Image *
+add_shrunken_image(imf)
+ImageFamily *imf;
+{
+    int numbytes, numbytes2, x, y, sum;
+    Image *img, *img2;
+
+    if (imf == NULL)
+      return NULL;
+    img = smallest_image(imf);
+    /* Don't try to shrink tiles or already-shrunken images or very
+       small images. */
+    if (img->istile || img->synthetic || img->w <= 8 || img->h <= 8)
+      return NULL;
+    img2 = get_img(imf, img->w / 2, img->h / 2);
+    /* Chances are that any embedded subimage will become unrecognizable,
+       so leave embedname empty. */
+    if (img2->embedx > 0)
+      img2->embedx = img->embedx / 2;
+    if (img2->embedy > 0)
+      img2->embedy = img->embedy / 2;
+    if (img2->embedw > 0)
+      img2->embedw = img->embedw / 2;
+    if (img2->embedh > 0)
+      img2->embedh = img->embedh / 2;
+    img2->pixelsize = img->pixelsize;
+    img2->palette = img->palette;
+    img2->rawpalette = img->rawpalette;
+    img2->numcolors = img->numcolors;
+    img2->synthetic = TRUE;
+    if (img->rawcolrdata == NULL && img->colrdata != lispnil) {
+	numbytes = img->h * computed_rowbytes(img->w, img->pixelsize);
+	img->rawcolrdata = xmalloc(numbytes);
+	interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
+    }
+    if (img->rawcolrdata != NULL) {
+	numbytes2 = img2->h * computed_rowbytes(img2->w, img2->pixelsize);
+	img2->rawcolrdata = xmalloc(numbytes2);
+    }
+    numbytes = img->h * computed_rowbytes(img->w, 1);
+    numbytes2 = img2->h * computed_rowbytes(img2->w, 1);
+    /* Ensure that binary version of mono image exists. */
+    /* (should make into sep. routine) */
+    if (img->rawmonodata == NULL && img->monodata != lispnil) {
+	img->rawmonodata = xmalloc(numbytes);
+	interp_bytes(img->monodata, numbytes, img->rawmonodata, 0);
+    }
+    if (img->rawmonodata != NULL)
+      img2->rawmonodata = xmalloc(numbytes2);
+    /* Ensure that binary version of mask exists. */
+    if (img->rawmaskdata == NULL && img->maskdata != lispnil) {
+	img->rawmaskdata = xmalloc(numbytes);
+	interp_bytes(img->maskdata, numbytes, img->rawmaskdata, 0);
+    }
+    if (img->rawmaskdata != NULL)
+      img2->rawmaskdata = xmalloc(numbytes2);
+    /* Scan through the new image, computing each pixel separately. */
+    for (x = 0; x < img2->w; ++x) {
+	for (y = 0; y < img2->h; ++y) {
+	    if (img2->rawcolrdata != NULL) {
+		/* (should choose most common or else average colors) */
+		int vals[4];
+		vals[0] = image_pixel_at(img, K_COLR_, 2 * x,     2 * y);
+		vals[1] = image_pixel_at(img, K_COLR_, 2 * x + 1, 2 * y);
+		vals[2] = image_pixel_at(img, K_COLR_, 2 * x,     2 * y + 1);
+		vals[3] = image_pixel_at(img, K_COLR_, 2 * x + 1, 2 * y + 1);
+		sum = vals[0];
+		if (vals[1] == vals[2] || vals[1] == vals[3])
+		  sum = vals[1];
+		else if (vals[2] == vals[3])
+		  sum = vals[2];
+		set_image_pixel_at(img2, K_COLR_, x, y, sum);
+	    }
+	    /* Add to the mono image if 2 or more of the 4 original
+	       bits are on. */
+	    if (img2->rawmonodata != NULL) {
+		sum = 0;
+		sum += image_pixel_at(img, K_MONO_, 2 * x,     2 * y);
+		sum += image_pixel_at(img, K_MONO_, 2 * x + 1, 2 * y);
+		sum += image_pixel_at(img, K_MONO_, 2 * x,     2 * y + 1);
+		sum += image_pixel_at(img, K_MONO_, 2 * x + 1, 2 * y + 1);
+		sum = (sum >= 2 ? 1 : 0);
+		set_image_pixel_at(img2, K_MONO_, x, y, sum);
+	    }
+	    /* Add to the mask if any of the 4 original mask bits are on. */
+	    if (img2->rawmaskdata != NULL) {
+		if (   image_pixel_at(img, K_MASK_, 2 * x,     2 * y)
+		    || image_pixel_at(img, K_MASK_, 2 * x + 1, 2 * y)
+		    || image_pixel_at(img, K_MASK_, 2 * x,     2 * y + 1)
+		    || image_pixel_at(img, K_MASK_, 2 * x + 1, 2 * y + 1))
+		  set_image_pixel_at(img2, K_MASK_, x, y, 1);
+	    }
+	}
+    }
+    compute_image_bbox(img2);
+    if (imf_interp_hook)
+      (*imf_interp_hook)(imf, img2, FALSE);
+    return img2;
+}
+
+/* Add an image twice the size of the largest image in the family, if
+   the family does not already include a large image. */
+
+static Image *
+add_magnified_image(imf)
+ImageFamily *imf;
+{
+    int numbytes, numbytes2, x, y, x2, y2;
+    Image *img, *img2;
+
+    if (imf == NULL)
+      return NULL;
+    img = largest_image(imf);
+    if (img->istile || img->synthetic || img->w >= 64 || img->h >= 64)
+      return NULL;
+    img2 = get_img(imf, img->w * 2, img->h * 2);
+    img2->embedname = img->embedname;
+    if (img2->embedx > 0)
+      img2->embedx = img->embedx * 2;
+    if (img2->embedy > 0)
+      img2->embedy = img->embedy * 2;
+    if (img2->embedw > 0)
+      img2->embedw = img->embedw * 2;
+    if (img2->embedh > 0)
+      img2->embedh = img->embedh * 2;
+    img2->pixelsize = img->pixelsize;
+    img2->palette = img->palette;
+    img2->rawpalette = img->rawpalette;
+    img2->numcolors = img->numcolors;
+    img2->synthetic = TRUE;
+    if (img->rawcolrdata == NULL && img->colrdata != lispnil) {
+	numbytes = img->h * computed_rowbytes(img->w, img->pixelsize);
+	img->rawcolrdata = xmalloc(numbytes);
+	interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
+    }
+    if (img->rawcolrdata != NULL) {
+	numbytes2 = img2->h * computed_rowbytes(img2->w, img2->pixelsize);
+	img2->rawcolrdata = xmalloc(numbytes2);
+    }
+    numbytes = img->h * computed_rowbytes(img->w, 1);
+    numbytes2 = img2->h * computed_rowbytes(img2->w, 1);
+    if (img->rawmonodata == NULL && img->monodata != lispnil) {
+	img->rawmonodata = xmalloc(numbytes);
+	interp_bytes(img->monodata, numbytes, img->rawmonodata, 0);
+    }
+    if (img->rawmonodata != NULL)
+      img2->rawmonodata = xmalloc(numbytes2);
+    if (img->rawmaskdata == NULL && img->maskdata != lispnil ) {
+	img->rawmaskdata = xmalloc(numbytes);
+	interp_bytes(img->maskdata, numbytes, img->rawmaskdata, 0);
+    }
+    if (img->rawmaskdata != NULL)
+      img2->rawmaskdata = xmalloc(numbytes2);
+    for (x = 0; x < img2->w; ++x) {
+	for (y = 0; y < img2->h; ++y) {
+	    x2 = x >> 1;  y2 = y >> 1;
+	    if (img->rawcolrdata != NULL) {
+		set_image_pixel_at(img2, K_COLR_, x, y,
+				   image_pixel_at(img, K_COLR_, x2, y2));
+	    }
+	    if (img->rawmonodata != NULL) {
+		set_image_pixel_at(img2, K_MONO_, x, y,
+				   image_pixel_at(img, K_MONO_, x2, y2));
+	    }
+	    if (img->rawmaskdata != NULL) {
+		set_image_pixel_at(img2, K_MASK_, x, y,
+				   image_pixel_at(img, K_MASK_, x2, y2));
+	    }
+	}
+    }
+    compute_image_bbox(img2);
+    if (imf_interp_hook)
+      (*imf_interp_hook)(imf, img2, FALSE);
+    return img2;
+}
+
 /* Given a list of strings, interpret the hex digits and put the
    results at the given address. */
 
@@ -716,7 +962,7 @@ char *name;
     return imp;
 }
 
-/* Find the image Palette of the given name, if it exists. */
+/* Find the image palette of the given name, if it exists. */
 
 ImagePalette *
 find_imp(name)
@@ -732,43 +978,44 @@ char *name;
 }
 
 
-/* Try to find the best of multiple images for the given bounding box. */
-/* Don't return anything that won't fit in min space. */
+/* Try to find the best of multiple images for the given bounding box.
+   Don't return anything that won't fit in min space. */
 
 Image *
 best_image(imf, w, h)
 ImageFamily *imf;
 int w, h;
 {
-    Image *img, *best = NULL, *fallback = NULL;
+    Image *img, *best = NULL, *fallback;
 
-    if (imf == NULL)
+    if (imf == NULL || imf->images == NULL)
       return NULL;
-    for (img = imf->images; img != NULL; img = img->next) {
-	/* Exact matches need no further searching. */
-	/* The istile test is a hack for the general problem here, which
-	   is that a high-quality color image might look better, even after
-	   scaling, than a b/w image that is exactly the right size. */
-	if (w == img->w && h == img->h && !img->istile) {
-	    return img;
-	} else if (between(img->minw, w, img->maxw)
-		   && between(img->minh, h, img->maxh)) {
-	    /* Image is plausible - go for the largest one that will fit. */
-	    if (!best || (img->w > best->w && img->h > best->h))
+    if (imf->images->istile) {
+	/* For tiles, always use the largest available. */
+	/* (should test depth also...) */
+	for_all_images(imf, img) {
+	    if (best == NULL || (img->w > best->w && img->h > best->h))
 	      best = img;
-	} else if (w >= img->minw && h >= img->minh) {
-	    /* Image is not really big enough, but keep as a fallback. */
-	    /* Prefer the largest fallback possible. */
-	    if (!fallback || (img->w > fallback->w && img->h > fallback->h))
-	      fallback = img;
-	} else if (img->istile) {
-	    /* Image is not really appropriate, but keep as a fallback. */
-	    /* Prefer the largest fallback possible. */
-	    if (!fallback || (img->w > fallback->w && img->h > fallback->h))
-	      fallback = img;
+	}
+    } else {
+	for_all_images(imf, img) {
+	    /* Exact matches need no further searching. */
+	    if (w == img->w && h == img->h)
+	      return img;
+	    if (best == NULL
+		|| (img->w < w && img->h < h
+		    && img->w > best->w && img->h > best->h))
+	      best = img;
+	}
+	if (best->w > w && best->h > h) {
+	    best = add_shrunken_image(imf);
+	} else if (best->w <= (w >> 1) && best->h <= (h >> 1)) {
+	    fallback = add_magnified_image(imf);
+	    if (fallback != NULL)
+	      best = fallback;
 	}
     }
-    return (best ? best : fallback);
+    return best;
 }
 
 Image *
@@ -779,14 +1026,26 @@ ImageFamily *imf;
 
     if (imf == NULL)
       return NULL;
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
 	if (smallest == NULL || (img->w < smallest->w && img->h < smallest->h))
 	  smallest = img;
     }
-    if (smallest != NULL) {
-	return smallest;
+    return smallest;
+}
+
+Image *
+largest_image(imf)
+ImageFamily *imf;
+{
+    Image *img, *largest = NULL;
+
+    if (imf == NULL)
+      return NULL;
+    for_all_images(imf, img) {
+	if (largest == NULL || (img->w > largest->w && img->h > largest->h))
+	  largest = img;
     }
-    return NULL;
+    return largest;
 }
 
 /* Compute the right location for the given emblem and unit images. */
@@ -909,7 +1168,7 @@ ImageFamily *imf;
     if (imf->name == NULL) {
 	return;
     }
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
 	/* Check consistency of Lisp and binary data. */
 	if (img->colrdata != lispnil && img->rawcolrdata) {
 	    /* (should add color image comparison) */
@@ -1010,7 +1269,7 @@ ImageFamily *imf;
 	fprintlisp(fp, imf->notes);
 	fprintf(fp, "))\n");
     }
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
 	if (img->monodata != lispnil
 	    || img->maskdata != lispnil
 	    || img->colrdata != lispnil
@@ -1018,6 +1277,9 @@ ImageFamily *imf;
 	    || img->rawmaskdata != NULL
 	    || img->rawcolrdata != NULL
 	    || (img->w == 1 && img->h == 1)) {
+	    /* Skip over synthesized images. */
+	    if (img->synthetic && !write_synthetic_also)
+	      continue;
 	    fprintf(fp, "(%s \"%s\"", keyword_name(K_IMF), imf->name);
 	    fprintf(fp, " (");
 	    fprintf(fp, "(%d %d", img->w, img->h);
