@@ -10,7 +10,11 @@ any later version.  See the file COPYING.  */
 #include "kpublic.h"
 #include "macconq.h"
 
+#ifdef NEW_HEADERS
+#include <ColorPicker.h>
+#else
 #include <Picker.h>
+#endif
 
 // --- EXTERN DEFINED IN MACDRAW.C  ---------------------------------------------
 
@@ -530,139 +534,6 @@ write_xcol(void)
 	}
 }
 
-/* The purpose of these 4 functions is to ensure the current cicn set is properly
-   initialized through a dummy PlotCIcon call. This dummy call is absolutely
-   essential if one wishes to speed up and/or modify plotting by direct access
-   of the cicn PixMap and BitMap, as is now done in draw_unit_image and draw_side_emblem. 
-   The reason for this is that PlotCIcon not only plots the cicn, but also sets up
-   a pointer to the PixMap data at offset zero. If this has not been done at least
-   once, CopyBits cannot handle the cicn. Moreover, since this is a pointer and not
-   a handle, it gets trashed whenever the cicn or PixMap data is moved in memory.
-   This is why a dummy call to PlotCIcon is needed before every access of the PixMap.
-   To avoid this, the cicn, PixMap and CTable are now locked in memory right after
-   the first (and only) PlotCIcon call. To prevent fragmentation, the handles are
-   also moved to the top of the heap zone before this is done. */
-
-/* Init the unit cicns and lock them at the top of the heap zone. */
-
-void
-init_unit_cicns()
-{
-	int u;
-	CIconHandle cicnhandle;
-	MacImage *macimg;
-	Image *img;
-	
-	for_all_unit_types(u) {
-		for (img = uimages[u]->images; img != NULL; img = img->next) {
-			if (img && img->hook) {
-				macimg = (MacImage *) img->hook;
-				if (macimg->colricon != NULL) {
-					cicnhandle = (CIconHandle) macimg->colricon;
-
-					/* Hang 'em High! */
-					HLockHi((Handle) cicnhandle);
-					HLockHi((Handle) (*cicnhandle)->iconData);
-					HLockHi((Handle) (*cicnhandle)->iconPMap.pmTable);
-
-					/* Initialize PixMap pointer by this dummy call. */
-					PlotCIcon(NULL, cicnhandle);
-
-					/* Lock down stuff that got unlocked by PlotCIcon. */
-					HLock((Handle) (*cicnhandle)->iconData);
-					HLock((Handle) cicnhandle);
-				}
-			}
-		}
-	}
-}
-
-/* Trash the unit cicns. */
-
-void
-dispose_unit_cicns()
-{
-	int u;
-	CIconHandle cicnhandle;
-	MacImage *macimg;
-	Image *img;
-	
-	for_all_unit_types(u) {
-		for (img = uimages[u]->images; img != NULL; img = img->next) {
-			if (img && img->hook) {
-				macimg = (MacImage *) img->hook;
-				if (macimg->colricon != NULL) {
-					cicnhandle = (CIconHandle) macimg->colricon;
-					DisposHandle((Handle) (*cicnhandle)->iconPMap.pmTable);
-					DisposHandle((Handle) (*cicnhandle)->iconData);
-					DisposHandle((Handle) cicnhandle);
-					macimg->colricon = nil;
-				}
-			}
-		}
-	}
-}
-
-/* Init the emblem cicns and lock them at the top of the heap zone. */
-
-void
-init_emblem_cicns()
-{
-	Side *side;
-	CIconHandle cicnhandle;
-	MacImage *macimg;
-	Image *img;
-	
-	for_all_sides_plus_indep(side) {
-		for (img = eimages[side_number(side)]->images; img != NULL; img = img->next) {
-			if (img && img->hook) {
-				macimg = (MacImage *) img->hook;
-				if (macimg->colricon != NULL) {
-					cicnhandle = (CIconHandle) macimg->colricon;
-
-					/* Hang 'em High! */
-					HLockHi((Handle) cicnhandle);
-					HLockHi((Handle) (*cicnhandle)->iconData);
-					HLockHi((Handle) (*cicnhandle)->iconPMap.pmTable);
-
-					/* Initialize PixMap pointer by this dummy call. */
-					PlotCIcon(NULL, cicnhandle);
-
-					/* Lock down stuff that got unlocked by PlotCIcon. */
-					HLock((Handle) (*cicnhandle)->iconData);
-					HLock((Handle) cicnhandle);
-				}
-			}
-		}
-	}
-}
-
-/* Trash the emblem cicns, thus clearing up the high part of the heap. */
-
-void
-dispose_emblem_cicns()
-{
-	Side *side;
-	CIconHandle cicnhandle;
-	MacImage *macimg;
-	Image *img;
-	return;
-	for_all_sides_plus_indep(side) {
-		for (img = eimages[side_number(side)]->images; img != NULL; img = img->next) {
-			if (img && img->hook) {
-				macimg = (MacImage *) img->hook;
-				if (macimg->colricon != NULL) {
-					cicnhandle = (CIconHandle) macimg->colricon;
-					DisposHandle((Handle) (*cicnhandle)->iconPMap.pmTable);
-					DisposHandle((Handle) (*cicnhandle)->iconData);
-					DisposHandle((Handle) cicnhandle);
-					macimg->colricon = nil;
-				}
-			}
-		}
-	}
-}
-
 // --- COLOR MANAGING DIALOGS ----------------------------------------------------
 
 /* The following three dialogs are used by the Colors menu and also by Preferences : Colorschemes */
