@@ -220,6 +220,8 @@ main()
 	init_fonts();
 	init_menus();
 	init_rects();
+	imf_load_hook = mac_load_imf;
+	imf_interp_hook = mac_interp_imf;
 	/* Put the Xconq kernel into a known state. */
 	clear_game_modules();
 	init_data_structures();
