@@ -22,7 +22,11 @@ any later version.  See the file COPYING.  */
 #ifndef __MWERKS__ /* actually, "old headers" */
 #include <Values.h>
 #endif
+#ifdef NEW_HEADERS
+#include <MacTypes.h>
+#else
 #include <Types.h>
+#endif
 #include <Resources.h>
 #include <QuickDraw.h>
 #include <Fonts.h>
@@ -31,10 +35,16 @@ any later version.  See the file COPYING.  */
 #include <Menus.h>
 #include <TextEdit.h>
 #include <Dialogs.h>
+#ifndef NEW_HEADERS
 #include <Desk.h>
+#endif
 #include <Scrap.h>
 #include <ToolUtils.h>
+#ifdef NEW_HEADERS
+#include <MacMemory.h>
+#else
 #include <Memory.h>
+#endif
 #include <SegLoad.h>
 #include <Files.h>
 #include <Folders.h>
@@ -51,7 +61,11 @@ any later version.  See the file COPYING.  */
 
 #include <QDOffscreen.h>
 
+#ifdef NEW_HEADERS
+#include <Gestalt.h>
+#else
 #include <GestaltEqu.h>
+#endif
 #include <PPCToolbox.h>
 #include <AppleEvents.h>
 
