@@ -66,7 +66,7 @@ static Image **best_unseen_images = NULL;
 void
 draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int mod, int emblem)
 {
-	int ex, ey, ew, eh, bx, by, bw, bh, overlap;
+	int ex, ey, ew, eh, bx, by, bw, bh, overlap, desperate, sx2, sy2;
 	char *ename;
 	Rect srcrect, imagerect;
 	CIconHandle cicnhandle;
@@ -121,12 +121,28 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 		BackColor(whiteColor);
 		return;
 	}
+	desperate = FALSE;
 
 	/* Get best unit image from family */
 	uimg = best_image(uimages[u], sw, sh);
 	if (uimg && uimg->hook) {
 		macuimg = (MacImage *) uimg->hook;
 
+		if (uimg->w < sw || uimg->h < sh) {
+			sx2 = sx + (sw - uimg->w) / 2;  sy2 = sy + (sh - uimg->h) / 2;
+			/* Only change the size of the rectangle being drawn if it's
+			   smaller than what was passed in. */
+			if (uimg->w < sw) {
+			    sx = sx2;
+			    sw = uimg->w;
+			}
+			if (uimg->h < sh) {
+			    sy = sy2;
+			    sh = uimg->h;
+			}
+			SetRect(&imagerect, sx, sy, sx + sw, sy + sh);
+		}
+	
 		portbits = QD(thePort)->portBits;
 
 		/* Use pict resource if available */
@@ -254,8 +270,6 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 		/* Use SICN if no pict, cicn or ICON exist */
 		} else if (macuimg->monosicn != nil) {
 			SetRect(&srcrect, 0, 0, 16, 16);
-			if (sw > 64)
-			  InsetRect(&imagerect, sw / 4, sh / 4);
 			bm.rowBytes = 2;
 			bm.bounds = srcrect;
 
@@ -285,19 +299,13 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 
 		/* Use box if no pict, cicn, ICON or SICN was found */
 		} else {
-			FillRect(&imagerect, QDPat(black));
-			/* Optionally use a second color to plot the right half of the square */
-			if (((curmap && curmap->sidecolors) || (curlist && curlist->sidecolors)) 
-				&& half_icon_color[e] && split_icon[e]) {
-				RGBForeColor(&(get_sideColor(e, half_icon_color[e])));
-				/* Restrict plotting to right half of image */
-				imagerect.left = imagerect.left + sw / 2;
-				FillRect(&imagerect, QDPat(black));
-			}
+			desperate = TRUE;
 		}
-
-	/* Also use box if no image is defined at all */ 
 	} else {
+		desperate = TRUE;
+	}
+	/* If nothing is working, draw a simple black box. */
+	if (desperate) {
 		FillRect(&imagerect, QDPat(black));
 		/* Optionally use a second color to plot the right half of the square */
 		if (((curmap && curmap->sidecolors) || (curlist && curlist->sidecolors)) 
@@ -334,7 +342,7 @@ draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int
 void
 draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style)
 {
-	int actualw, actualh;
+	int ex2, ey2;
 	Rect srcrect, imagerect, shadowrect;
 	CIconHandle cicnhandle;
 	Image *eimg;
@@ -344,29 +352,26 @@ draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style
 	PixMap pm;					/* icon PixMap */
 
 	/* Filter out very small images. */
-	if (ew <= 2)
+	if (ew <= 4)
 	  return;
 
 	/* Get best emblem image from family. */
 	eimg = best_image(eimages[e], ew, eh);
 
-	/* Get destination rectangle. */
-	imagerect.top = ey;  
-	imagerect.left = ex;
-	actualw = eimg->w;  
-	actualh = eimg->h;
-	if (ew >= actualw * 2 && eh >= actualh * 2) {
-		actualw *= 2;  actualh *= 2;
+	if (eimg->w < ew || eimg->h < eh) {
+		ex2 = ex + (ew - eimg->w) / 2;  ey2 = ey + (eh - eimg->h) / 2;
+		/* Only change the size of the rectangle being drawn if it's
+		   smaller than what was passed in. */
+		if (eimg->w < ew) {
+		    ex = ex2;
+		    ew = eimg->w;
+		}
+		if (eimg->h < eh) {
+		    ey = ey2;
+		    eh = eimg->h;
+		}
 	}
-	if (ew >= actualw * 2 && eh >= actualh * 2) {
-		actualw *= 2;  actualh *= 2;
-	}
-	if (actualw < ew)
-	  ew = actualw;
-	if (actualh < eh)
-	  eh = actualh;
-	imagerect.bottom = imagerect.top + eh;  
-	imagerect.right = imagerect.left + ew;
+	SetRect(&imagerect, ex, ey, ex + ew, ey + eh);
 
 	/* If an image is available, display it, otherwise do nothing. */
 	if (eimg && eimg->hook) {
@@ -384,7 +389,7 @@ draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style
 				   && (minscreendepth > 1
 				   || (maceimg->monoicon == nil && maceimg->monosicn == nil))) {
 
-			SetRect(&srcrect, 0, 0, eimg->w, eimg->h);
+			SetRect(&srcrect, 0, 0, ew, eh);
 			cicnhandle = (CIconHandle) maceimg->colricon;
 		
 			if (style == shadow_emblem) {
