@@ -12,10 +12,11 @@ any later version.  See the file COPYING.  */
 
 /* Needed for changing screen depths and handling the control strip */
 
-#include <palettes.h>
+#include <Palettes.h>
 #ifdef CONTROL_STRIP
 #include <ControlStrip.h>
-#include "applestuff.h"	/* Apple control strip routine descriptors */
+/* Apple control strip routine descriptors */
+#include "applestuff.h"
 #endif
 
 // --- USED TO DEBUG OFFSCREEN GRAPHICS ----------------------------------------
@@ -199,17 +200,17 @@ scroll_map_window(Map *map, int dx, int dy)
 	map->vp->sy = newsy;
 	set_content_rect(map);
 
-	/* Always update if dy take us outside the gworld */
-	if ( dy > 0 && map->offsety + dy >   map->bufy ||
-	       dy < 0 && map->offsety + dy < -map->bufy ) {
-		redraw = TRUE;
-	}		
-	/* Handle dx for wrapped gworlds. Never update */
+	/* Handle dx for wrapped gworlds.  Never update. */
 	if (area.xwrap && map->vp->sxmax <= portrect.right - portrect.left) {
 		redraw = FALSE; 
-	/* Handle dx in other cases. Update if stepping outside */
-	} else if (dx > 0 && map->offsetx + dx >   map->bufx || 
-	    	      dx < 0 && map->offsetx + dx < -map->bufx ) {
+	} else if ((dx > 0 && map->offsetx + dx > map->bufx)
+			   || (dx < 0 && map->offsetx + dx < -map->bufx)) {
+		/* Handles dx in other cases.  Update if stepping outside. */
+		redraw = TRUE;
+	}		
+	/* Always update if dy take us outside the gworld */
+	if ((dy > 0 && map->offsety + dy > map->bufy)
+		|| (dy < 0 && map->offsety + dy < -map->bufy)) {
 		redraw = TRUE;
 	}		
 
@@ -733,7 +734,7 @@ auto_scroll_left(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, 0);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((long *) keyMap);
 		keyDown = keyMap[0x56 >> 3] >> (0x56 & 7) & 1;	/* keypad 4 */
 	}
 }
@@ -748,7 +749,7 @@ auto_scroll_right(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, 0);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((long *) keyMap);
 		keyDown = keyMap[0x58 >> 3] >> (0x58 & 7) & 1;	/* keypad 6 */
 	}
 }
@@ -763,7 +764,7 @@ auto_scroll_up(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 0, -4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((long *) keyMap);
 		keyDown = keyMap[0x5B >> 3] >> (0x5B & 7) & 1;	/* keypad 8 */
 	}
 }
@@ -778,7 +779,7 @@ auto_scroll_down(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 0, 4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((long *) keyMap);
 		keyDown = keyMap[0x54 >> 3] >> (0x54 & 7) & 1;	/* keypad 2 */
 	}
 }
@@ -793,7 +794,7 @@ auto_scroll_down_left(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, 4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((long *) keyMap);
 		keyDown = keyMap[0x53 >> 3] >> (0x53 & 7) & 1;	/* keypad 1 */
 	}
 }
@@ -808,7 +809,7 @@ auto_scroll_down_right(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, 4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((long *) keyMap);
 		keyDown = keyMap[0x55 >> 3] >> (0x55 & 7) & 1;	/* keypad 3 */
 	}
 }
@@ -823,7 +824,7 @@ auto_scroll_up_left(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, -4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((long *) keyMap);
 		keyDown = keyMap[0x59 >> 3] >> (0x59 & 7) & 1;	/* keypad 7 */
 	}
 }
@@ -838,7 +839,7 @@ auto_scroll_up_right(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, -4);
-		GetKeys((unsigned long *) keyMap);
+		GetKeys((long *) keyMap);
 		keyDown = keyMap[0x5C >> 3] >> (0x5C & 7) & 1;	/* keypad 9 */
 	}
 }
