@@ -1,5 +1,5 @@
 /* Image families for the Mac interface to Xconq.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,8 +17,16 @@ any later version.  See the file COPYING.  */
 #ifdef THINK_C
 #include <MacHeaders>
 #else /* assume MPW */
+#ifdef NEW_HEADERS
+#include <MacTypes.h>
+#else
 #include <Types.h>
+#endif
+#ifdef NEW_HEADERS
+#include <MacMemory.h>
+#else
 #include <Memory.h>
+#endif
 #include <Resources.h>
 #include <Quickdraw.h>
 #endif /* THINK_C */
@@ -29,12 +37,16 @@ any later version.  See the file COPYING.  */
 
 extern int hasColorQD;
 
+extern PixPatHandle mac_create_ppat(Image *img);
 extern CIconHandle mac_create_cicn(Image *img);
+extern void mac_init_cicn(Image *img);
 extern void mac_add_cicn(ImageFamily *imf, Handle handle);
 extern void mac_add_ppat(ImageFamily *imf, Handle handle);
 
 static MacImage *init_mac_image(Image *img);
+static void mac_interp_image(ImageFamily *imf, Image *img, int force);
 static void mac_interp_bytes(Obj *datalist, int numbytes, Handle desthandle, int jump);
+static void mac_copy_bytes(char *data, int numbytes, Handle desthandle, int jump, int rowbytes);
 static CTabHandle interp_ctab(Obj *palette);
 static CTabHandle synth_ctab(void);
 static void convert_ppat(Image *img, PixPatHandle colrpat);
@@ -69,7 +81,7 @@ get_mac_image(Image *img)
 /* This tries to fill in the given image family from various resources.  The order
    should be "best first", so as to improve search time. */
 
-void
+ImageFamily *
 mac_load_imf(ImageFamily *imf)
 {
     int w, h, i, rsize, startlineno = 0, endlineno = 0;
@@ -85,7 +97,7 @@ mac_load_imf(ImageFamily *imf)
 
 	/* Can't do anything without a name for the image family. */
 	if (imf == NULL || imf->name == NULL)
-	  return;
+	  return NULL;
 	c2p(imf->name, namestr);
 	/* The name of the mask is always formed by appending " mask". */
 	sprintf(tmpstrbuf, "%s mask", imf->name);
@@ -109,8 +121,6 @@ mac_load_imf(ImageFamily *imf)
 	if (pichandle != nil) {
 		img = get_img(imf, 16, 16); /* should get real bounds */
 		if (img != NULL) {
-			img->minw = img->minh = 8;
-			img->maxw = img->maxh = 9999;
 			macimg = get_mac_image(img);
 			/* (should distinguish mono and color picts somehow) */
 			macimg->monopict = pichandle;
@@ -138,8 +148,6 @@ mac_load_imf(ImageFamily *imf)
 	if (iconhandle != nil) {
 		img = get_img(imf, 32, 32);
 		if (img != NULL) {
-			img->minw = img->minh = 8;
-			img->maxw = img->maxh = 128;
 			macimg = get_mac_image(img);
 			macimg->monoicon = iconhandle;
 			/* Look for a mask too. */
@@ -154,8 +162,6 @@ mac_load_imf(ImageFamily *imf)
 	if (sicnhandle != nil) {
 		img = get_img(imf, 16, 16);
 		if (img != NULL) {
-			img->minw = img->minh = 8;
-			img->maxw = img->maxh = 64;
 			macimg = get_mac_image(img);
 			/* Image itself is just the first 32 bytes, mask is second 32 if present. */
 			macimg->monosicn = sicnhandle;
@@ -195,8 +201,6 @@ mac_load_imf(ImageFamily *imf)
 	if (pathandle != nil) {
 		img = get_img(imf, 8, 8);
 		if (img != NULL) {
-			img->minw = img->minh = 1;
-			img->maxw = img->maxh = 9999;
 			img->istile = TRUE;
 			macimg = get_mac_image(img);
 			if (macimg->patdefined) {
@@ -229,12 +233,13 @@ mac_load_imf(ImageFamily *imf)
 			}
 		}
 	}
+	return imf;
 }
 
 void
 mac_add_cicn(ImageFamily *imf, Handle handle)
 {
-    int n, i, w, h;
+    int n, i, w, h, newcicn;
     Image *img;
     MacImage *macimg;
     CIconHandle cicnhandle;
@@ -244,7 +249,7 @@ mac_add_cicn(ImageFamily *imf, Handle handle)
 
 	if (handle == nil)
 	  return;
-	HLock(handle);
+	newcicn = FALSE;
 	/* Need to get id so we can use special trap. */
 	GetResInfo(handle, &resid, &restype, resname);
 	cicnhandle = GetCIcon(resid);
@@ -254,14 +259,17 @@ mac_add_cicn(ImageFamily *imf, Handle handle)
 	w = bounds.right - bounds.left;  h = bounds.bottom - bounds.top;
 	img = get_img(imf, w, h);
 	if (img != NULL) {
-		img->minw = img->minh = 8;
-		img->maxw = img->maxh = 128;
 		macimg = get_mac_image(img);
-		macimg->colricon = (Handle) cicnhandle;
-		/* Mask is built in, don't need to load separately. */
+		if (!macimg->cicn_inited) {
+			macimg->colricon = (Handle) cicnhandle;
+			newcicn = TRUE;
+			/* Mask is built in, don't need to load separately. */
+		}
 	}
-	HUnlock((Handle) cicnhandle);
-	HUnlock(handle);
+	/* We want to unlock the cicn now, and lock it high later on;
+	   but not if it was already processed. */
+	if (newcicn)
+	  HUnlock((Handle) cicnhandle);
 }
 
 void
@@ -313,8 +321,6 @@ mac_add_ppat(ImageFamily *imf, Handle handle)
 	img = get_img(imf, w, h);
 	if (img != NULL) {
 		/* Indicate that we can use this pattern for any size area. */
-		img->minw = img->minh = 1;
-		img->maxw = img->maxh = 9999;
 		img->istile = TRUE;
 		macimg = get_mac_image(img);
 		if (w == 1 && h == 1) {
@@ -330,9 +336,6 @@ mac_add_ppat(ImageFamily *imf, Handle handle)
 			/* A ppat always has an 8x8 mono pattern defined. */
 			img = get_img(imf, 8, 8);
 			if (img != NULL) {
-				/* Indicate that we can use this pattern for any size area. */
-				img->minw = img->minh = 1;
-				img->maxw = img->maxh = 9999;
 				img->istile = TRUE;
 				macimg = get_mac_image(img);
 				/* Set the monopat. */
@@ -349,137 +352,146 @@ mac_add_ppat(ImageFamily *imf, Handle handle)
 /* Given an image family that already has data for it, in Lisp form,
    make platform-specific images. */
 
+ImageFamily *
+mac_interp_imf(imf, img, force)
+ImageFamily *imf;
+Image *img;
+int force;
+{
+	if (img == NULL) {
+		for_all_images(imf, img) {
+			mac_interp_image(imf, img, force);
+		}
+	} else {
+		mac_interp_image(imf, img, force);
+	}
+	return imf;
+}
+
 void
-mac_interp_imf(ImageFamily *imf)
+mac_interp_image(ImageFamily *imf, Image *img, int force)
 {
 	int w, h, i, numbytes, bitrowbytes, actualw, actualh, pixelsize, rowbytes;
 	int macrowbytes;
 	int monodone, colrdone, maskdone;
-	Image *img;
 	MacImage *macimg;
 	Handle sicnhandle, iconhandle, datahandle;
     PixPatHandle ppathandle;
     CIconHandle cicnhandle;
     PixMapHandle pmhandle;
 
-	for (img = imf->images; img != NULL; img = img->next) {
-		w = img->w;  h = img->h;
-		actualw = img->actualw;  actualh = img->actualh;
-		pixelsize = img->pixelsize;
-		rowbytes = computed_rowbytes(actualw, pixelsize);
-		macimg = get_mac_image(img);
-		/* Mono icons and masks are very similar; digest both here. */
-		monodone = colrdone = maskdone = FALSE;
-		if (w == 8 && h == 8 && img->monodata != lispnil && img->istile) {
-			/* Monochrome pattern. */
-			datahandle = NewHandle(8);
-			memset(*datahandle, 0, 8);
-			/* Read exactly 8 bytes. */
-			mac_interp_bytes(img->monodata, 8, datahandle, 0);
-			/* Fill in the monopat. */
-			for (i = 0; i < 8; ++i)
-			  SET_IMG_PAT(macimg, i, (*datahandle)[i]);
-			macimg->patdefined = TRUE;
-			/* Patterns have no masks, but defeat subsequent mask hacking. */
-			monodone = maskdone = TRUE;
-		}
-		if (w == 16 && h == 16 && img->monodata != lispnil) {
-			/* Shape is appropriate for a small icon, make one. */
-			sicnhandle = NewHandle(32);
-			memset(*sicnhandle, 0, 32);
-			/* Read exactly 32 bytes. */
-			mac_interp_bytes(img->monodata, 32, sicnhandle, 0);
-			macimg->monosicn = sicnhandle;
-			monodone = TRUE;
-		}
-		if (w == 16 && h == 16 && img->maskdata != lispnil) {
-			/* Shape is appropriate for a small icon mask, make one. */
-			sicnhandle = NewHandle(32);
-			memset(*sicnhandle, 0, 32);
-			/* Read exactly 32 bytes. */
-			mac_interp_bytes(img->maskdata, 32, sicnhandle, 0);
-			macimg->masksicn = sicnhandle;
-			maskdone = TRUE;
-		}
-		if (w == 32 && h == 32 && img->monodata != lispnil) {
-			/* Shape is appropriate for a standard icon, make one. */
-			iconhandle = NewHandle(128);
-			memset(*iconhandle, 0, 128);
-			/* Read exactly 128 bytes. */
-			mac_interp_bytes(img->monodata, 128, iconhandle, 0);
-			macimg->monoicon = iconhandle;
-			monodone = TRUE;
-		}
-		if (w == 32 && h == 32 && img->maskdata != lispnil) {
-			/* Shape is appropriate for a standard icon, make one. */
-			iconhandle = NewHandle(128);
-			memset(*iconhandle, 0, 128);
-			/* Read exactly 128 bytes. */
-			mac_interp_bytes(img->maskdata, 128, iconhandle, 0);
-			macimg->maskicon = iconhandle;
-			maskdone = TRUE;
-		}
-		/* A 1x1 image is just a color - make it into a small color pattern. */
-		if (hasColorQD && w == 1 && h == 1 && img->istile) {
-			/* Make a simple color pattern. */
-			ppathandle = NewPixPat();
-			pmhandle = (*ppathandle)->patMap;
-			SetRect(&((*pmhandle)->bounds), 0, 0, 8, 8);
-			(*pmhandle)->rowBytes = 1;
-			(*pmhandle)->pixelType = 0;
-			(*pmhandle)->pixelSize = 1;
-			(*pmhandle)->cmpCount = 1;
-			(*pmhandle)->cmpSize = pixelsize;
-			(*pmhandle)->pmTable = interp_ctab(img->palette);
-			datahandle = NewHandle(8);
-			(*ppathandle)->patData = datahandle;
-			HLock(datahandle);
-			memset(*datahandle, 0, 8);
-			HUnlock(datahandle);
-			(*ppathandle)->patXValid = -1;
-			macimg->colrpat = ppathandle;
-			colrdone = maskdone = TRUE;
-		}
-		/* A sufficiently small tiling color image can be a color pattern. */
-		if (hasColorQD && w <= 64 && h <= 64 && img->istile && img->colrdata != lispnil) {
-			/* Limit bits per pixel to reasonable values. */
-			if (pixelsize < 1 || pixelsize > 8)
-			  pixelsize = 8;
-			macrowbytes = mac_computed_rowbytes(actualw, pixelsize);
-			/* As a special exception, PixPats can have a rowbytes of 1. */
-			if (actualw <= 8 && pixelsize == 1)
-			  macrowbytes = 1;
-			/* Make a color pattern. */
-			ppathandle = NewPixPat();
-			pmhandle = (*ppathandle)->patMap;
-			SetRect(&((*pmhandle)->bounds), 0, 0, actualw, actualh);
-			(*pmhandle)->rowBytes = macrowbytes;
-			(*pmhandle)->pixelType = 0;
-			(*pmhandle)->pixelSize = pixelsize;
-			(*pmhandle)->cmpCount = 1;
-			(*pmhandle)->cmpSize = pixelsize;
-			(*pmhandle)->pmTable = interp_ctab(img->palette);
-			numbytes = actualh * rowbytes;
-			datahandle = NewHandle(numbytes);
-			(*ppathandle)->patData = datahandle;
-			mac_interp_bytes(img->colrdata, numbytes, datahandle, macrowbytes - rowbytes);
-			(*ppathandle)->patXValid = -1;
-			macimg->colrpat = ppathandle;
-			colrdone = maskdone = TRUE;
-		}
-		if (hasColorQD && w <= 64 && h <= 64 && !img->istile
-		    && (img->colrdata != lispnil
-		        || (macimg->monosicn == nil
-		            && macimg->masksicn == nil
-		            && macimg->monoicon == nil
-		            && macimg->maskicon == nil))) {
-		    macimg->colricon = (Handle) mac_create_cicn(img);
-			colrdone = TRUE;
-		}
-		if (!monodone && !colrdone) {
-			run_warning("%dx%d image of \"%s\" could not be interpreted", w, h, imf->name);
-		}
-    }
+	w = img->w;  h = img->h;
+	actualw = img->actualw;  actualh = img->actualh;
+	pixelsize = img->pixelsize;
+	rowbytes = computed_rowbytes(actualw, pixelsize);
+	macimg = get_mac_image(img);
+	/* Mono icons and masks are very similar; digest both here. */
+	monodone = colrdone = maskdone = FALSE;
+	if (img->monodata != lispnil && (img->rawmonodata == NULL || force)) {
+		numbytes = h * computed_rowbytes(w, 1);
+		img->rawmonodata = xmalloc(numbytes);
+		interp_bytes(img->monodata, numbytes, img->rawmonodata, 0);
+	}
+	if (w == 8 && h == 8 && img->rawmonodata != NULL && img->istile) {
+		/* Monochrome pattern. */
+		datahandle = NewHandle(8);
+		memset(*datahandle, 0, 8);
+		/* Read exactly 8 bytes. */
+		mac_copy_bytes(img->rawmonodata, 8, datahandle, 0, rowbytes);
+		/* Fill in the monopat. */
+		for (i = 0; i < 8; ++i)
+		  SET_IMG_PAT(macimg, i, (*datahandle)[i]);
+		macimg->patdefined = TRUE;
+		/* Patterns have no masks, but defeat subsequent mask hacking. */
+		monodone = maskdone = TRUE;
+	}
+	if (w == 16 && h == 16 && img->rawmonodata != NULL) {
+		/* Shape is appropriate for a small icon, make one. */
+		sicnhandle = NewHandle(32);
+		memset(*sicnhandle, 0, 32);
+		/* Read exactly 32 bytes. */
+		mac_copy_bytes(img->rawmonodata, 32, sicnhandle, 0, rowbytes);
+		macimg->monosicn = sicnhandle;
+		monodone = TRUE;
+	}
+	if (w == 32 && h == 32 && img->rawmonodata != NULL) {
+		/* Shape is appropriate for a standard icon, make one. */
+		iconhandle = NewHandle(128);
+		memset(*iconhandle, 0, 128);
+		/* Read exactly 128 bytes. */
+		mac_copy_bytes(img->rawmonodata, 128, iconhandle, 0, rowbytes);
+		macimg->monoicon = iconhandle;
+		monodone = TRUE;
+	}
+	if (img->maskdata != lispnil && (img->rawmaskdata == NULL || force)) {
+		numbytes = h * computed_rowbytes(w, 1);
+		img->rawmaskdata = xmalloc(numbytes);
+		interp_bytes(img->maskdata, numbytes, img->rawmaskdata, 0);
+	}
+	if (w == 16 && h == 16 && img->rawmaskdata != NULL) {
+		/* Shape is appropriate for a small icon mask, make one. */
+		sicnhandle = NewHandle(32);
+		memset(*sicnhandle, 0, 32);
+		/* Read exactly 32 bytes. */
+		mac_copy_bytes(img->rawmaskdata, 32, sicnhandle, 0, rowbytes);
+		macimg->masksicn = sicnhandle;
+		maskdone = TRUE;
+	}
+	if (w == 32 && h == 32 && img->rawmaskdata != NULL) {
+		/* Shape is appropriate for a standard icon, make one. */
+		iconhandle = NewHandle(128);
+		memset(*iconhandle, 0, 128);
+		/* Read exactly 128 bytes. */
+		mac_copy_bytes(img->rawmaskdata, 128, iconhandle, 0, rowbytes);
+		macimg->maskicon = iconhandle;
+		maskdone = TRUE;
+	}
+	/* A 1x1 image is just a color - make it into a small color pattern. */
+	if (hasColorQD && w == 1 && h == 1 && img->istile) {
+		/* Make a simple color pattern. */
+		ppathandle = NewPixPat();
+		pmhandle = (*ppathandle)->patMap;
+		SetRect(&((*pmhandle)->bounds), 0, 0, 8, 8);
+		(*pmhandle)->rowBytes = 1;
+		(*pmhandle)->pixelType = 0;
+		(*pmhandle)->pixelSize = 1;
+		(*pmhandle)->cmpCount = 1;
+		(*pmhandle)->cmpSize = pixelsize;
+		(*pmhandle)->pmTable = interp_ctab(img->palette);
+		datahandle = NewHandle(8);
+		(*ppathandle)->patData = datahandle;
+		HLock(datahandle);
+		memset(*datahandle, 0, 8);
+		HUnlock(datahandle);
+		(*ppathandle)->patXValid = -1;
+		macimg->colrpat = ppathandle;
+		colrdone = maskdone = TRUE;
+	}
+	if (img->colrdata != lispnil && (img->rawcolrdata == NULL || force)) {
+		numbytes = h * computed_rowbytes(w, img->pixelsize);
+		img->rawcolrdata = xmalloc(numbytes);
+		interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
+	}
+	/* A sufficiently small tiling color image can be a color pattern. */
+	if (hasColorQD && w <= 64 && h <= 64 && img->istile && img->rawcolrdata != NULL) {
+		macimg->colrpat = mac_create_ppat(img);
+		colrdone = maskdone = TRUE;
+	}
+	/* Finally, a color icon covers all other cases. */
+	if (hasColorQD && w <= 64 && h <= 64 && !img->istile
+	    && (img->rawcolrdata != NULL
+	        || (macimg->monosicn == nil
+	            && macimg->masksicn == nil
+	            && macimg->monoicon == nil
+	            && macimg->maskicon == nil))) {
+	    macimg->colricon = (Handle) mac_create_cicn(img);
+	    macimg->cicn_inited = FALSE;
+		mac_init_cicn(img);
+		colrdone = TRUE;
+	}
+	if (!monodone && !colrdone) {
+		run_warning("%dx%d image of \"%s\" could not be interpreted", w, h, imf->name);
+	}
 #if 0 /* broken */
     /* If we have a color pattern of random size, and an 8x8 mono pattern, then we should
        glue the mono pattern into the color pattern's mono slot.  This requires searching
@@ -500,15 +512,51 @@ mac_interp_imf(ImageFamily *imf)
 #endif
 }
 
+PixPatHandle
+mac_create_ppat(Image *img)
+{
+	int numbytes, actualw, actualh, pixelsize, rowbytes, macrowbytes;
+	Handle iconhandle, datahandle;
+    PixPatHandle ppathandle;
+    PixMapHandle pmhandle;
+
+	actualw = img->actualw;  actualh = img->actualh;
+	pixelsize = img->pixelsize;
+	rowbytes = computed_rowbytes(actualw, pixelsize);
+	/* Limit bits per pixel to reasonable values. */
+	if (pixelsize < 1 || pixelsize > 8)
+	  pixelsize = 8;
+	macrowbytes = mac_computed_rowbytes(actualw, pixelsize);
+	/* As a special exception, PixPats can have a rowbytes of 1. */
+	if (actualw <= 8 && pixelsize == 1)
+	  macrowbytes = 1;
+	/* Make a color pattern. */
+	ppathandle = NewPixPat();
+	pmhandle = (*ppathandle)->patMap;
+	SetRect(&((*pmhandle)->bounds), 0, 0, actualw, actualh);
+	(*pmhandle)->rowBytes = macrowbytes;
+	(*pmhandle)->pixelType = 0;
+	(*pmhandle)->pixelSize = pixelsize;
+	(*pmhandle)->cmpCount = 1;
+	(*pmhandle)->cmpSize = pixelsize;
+	(*pmhandle)->pmTable = interp_ctab(img->palette);
+	numbytes = actualh * rowbytes;
+	datahandle = NewHandle(numbytes);
+	(*ppathandle)->patData = datahandle;
+	mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+	(*ppathandle)->patXValid = -1;
+	return ppathandle;
+}
+
 /* Given an image, make a color icon out of its data. */
 
 CIconHandle
 mac_create_cicn(Image *img)
 {
-	int w, h, i, numbytes, bitrowbytes, macbitrowbytes, actualw, actualh, pixelsize, rowbytes, macrowbytes;
+	int w, h, i, numbytes, bitrowbytes, macbitrowbytes, actualw, actualh;
+	int pixelsize, rowbytes, macrowbytes;
 	MacImage *macimg;
 	Handle sicnhandle, iconhandle, datahandle, monohandle, maskhandle;
-    PixPatHandle ppathandle;
     CIconHandle cicnhandle;
     PixMapHandle pmhandle;
 
@@ -516,7 +564,7 @@ mac_create_cicn(Image *img)
 	actualw = img->actualw;  actualh = img->actualh;
 	pixelsize = img->pixelsize;
     /* If no color data in evidence, prepare to use mono or mask instead. */
-    if (img->colrdata == lispnil)
+    if (img->rawcolrdata == NULL)
 	  pixelsize = 1;
 	rowbytes = computed_rowbytes(actualw, pixelsize);
 	macrowbytes = mac_computed_rowbytes(actualw, pixelsize);
@@ -557,14 +605,29 @@ mac_create_cicn(Image *img)
 	(*cicnhandle)->iconData = datahandle;
 	/* Fill up the datahandle with the color data, or else use mono/mask data
 	   if the color data is missing. */
-	if (img->colrdata != lispnil) {
-		mac_interp_bytes(img->colrdata, numbytes, datahandle, macrowbytes - rowbytes);
-	} else if (img->monodata != lispnil) {
-		mac_interp_bytes(img->monodata, numbytes, datahandle, macrowbytes - rowbytes);
+	if (img->colrdata != lispnil || img->rawcolrdata != NULL) {
+		if (img->rawcolrdata == NULL) {
+			int xnumbytes = img->h * computed_rowbytes(img->w, img->pixelsize);
+			img->rawcolrdata = xmalloc(xnumbytes);
+			interp_bytes(img->colrdata, xnumbytes, img->rawcolrdata, 0);
+		}
+		mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+	} else if (img->monodata != lispnil || img->rawmonodata != NULL) {
+		if (img->rawmonodata == NULL) {
+			int xnumbytes = h * computed_rowbytes(w, 1);
+			img->rawmonodata = xmalloc(xnumbytes);
+			interp_bytes(img->monodata, xnumbytes, img->rawmonodata, 0);
+		}
+		mac_copy_bytes(img->rawmonodata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
 		/* Make an ersatz color table. */
 		(*cicnhandle)->iconPMap.pmTable = synth_ctab();
-	} else if (img->maskdata != lispnil) {
-		mac_interp_bytes(img->maskdata, numbytes, datahandle, macrowbytes - rowbytes);
+	} else if (img->maskdata != lispnil || img->rawmaskdata != NULL) {
+		if (img->rawmaskdata == NULL) {
+			int xnumbytes = h * computed_rowbytes(w, 1);
+			img->rawmaskdata = xmalloc(xnumbytes);
+			interp_bytes(img->maskdata, xnumbytes, img->rawmaskdata, 0);
+		}
+		mac_copy_bytes(img->rawmaskdata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
 		/* Make an ersatz color table. */
 		(*cicnhandle)->iconPMap.pmTable = synth_ctab();
 	} else {
@@ -583,13 +646,18 @@ mac_create_cicn(Image *img)
 		HLock(macimg->monosicn);
 		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *(macimg->monosicn), numbytes);
 		HUnlock(macimg->monosicn);
-	} else if (img->monodata != lispnil) {
+	} else if (img->monodata != lispnil || img->rawmonodata != NULL) {
 		/* (should use static temp space here?) */
 		monohandle = NewHandle(numbytes);
 		HLock(monohandle);
 		memset(*monohandle, 0, numbytes);
 		/* Read as many bytes as directed. */
-		mac_interp_bytes(img->monodata, numbytes, monohandle, macbitrowbytes - bitrowbytes);
+		if (img->rawmonodata == NULL) {
+			int xnumbytes = h * computed_rowbytes(w, 1);
+			img->rawmonodata = xmalloc(xnumbytes);
+			interp_bytes(img->monodata, xnumbytes, img->rawmonodata, 0);
+		}
+		mac_copy_bytes(img->rawmonodata, numbytes, monohandle, macbitrowbytes - bitrowbytes, bitrowbytes);
 		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *monohandle, numbytes);
 		HUnlock(monohandle);
 	} else {
@@ -610,16 +678,21 @@ mac_create_cicn(Image *img)
 		HLock(macimg->masksicn);
 		memcpy((char *) (*cicnhandle)->iconMaskData, *(macimg->masksicn), numbytes);
 		HUnlock(macimg->masksicn);
-	} else if (img->maskdata != lispnil) {
+	} else if (img->maskdata != lispnil || img->rawmaskdata != NULL) {
 		maskhandle = NewHandle(numbytes);
 		HLock(maskhandle);
 		memset(*maskhandle, 0, numbytes);
 		/* Read as many bytes as directed. */
-		mac_interp_bytes(img->maskdata, numbytes, maskhandle, macbitrowbytes - bitrowbytes);
+		if (img->rawmaskdata == NULL) {
+			int xnumbytes = h * computed_rowbytes(w, 1);
+			img->rawmaskdata = xmalloc(xnumbytes);
+			interp_bytes(img->maskdata, xnumbytes, img->rawmaskdata, 0);
+		}
+		mac_copy_bytes(img->rawmaskdata, numbytes, maskhandle, macbitrowbytes - bitrowbytes, bitrowbytes);
 		memcpy((char *) (*cicnhandle)->iconMaskData, *maskhandle, numbytes);
 		HUnlock(maskhandle);
 	} else {
-		/* No mask data, make it black. */
+		/* No mask data, make it be the entire area. */
 		maskhandle = NewHandle(numbytes);
 		HLock(maskhandle);
 		memset(*maskhandle, 0xff, numbytes);
@@ -638,6 +711,64 @@ mac_interp_bytes(Obj *datalist, int numbytes, Handle desthandle, int jump)
     HUnlock(desthandle);
 }
 
+static void
+mac_copy_bytes(char *data, int numbytes, Handle desthandle, int jump, int rowbytes)
+{
+	int i, j;
+
+    HLock(desthandle);
+    if (jump == 0) {
+		memcpy(*desthandle, data, numbytes);
+    } else {
+		j = 0;
+		for (i = 0; i < numbytes; ++i) {
+			(*desthandle)[i] = data[j++];
+			if (jump == 1) {
+				++i;
+				(*desthandle)[i] = 0;
+			}
+		}
+    }
+    HUnlock(desthandle);
+}
+
+/* This function makes dummy calls to PlotCIcon.  The dummy call is absolutely
+   essential if one wishes to speed up and/or modify plotting by direct access
+   of the cicn PixMap and BitMap, as is now done in draw_unit_image and draw_side_emblem. 
+   The reason for this is that PlotCIcon not only plots the cicn, but also sets up
+   a pointer to the PixMap data at offset zero. If this has not been done at least
+   once, CopyBits cannot handle the cicn. Moreover, since this is a pointer and not
+   a handle, it gets trashed whenever the cicn or PixMap data is moved in memory.
+   This is why a dummy call to PlotCIcon is needed before every access of the PixMap.
+   To avoid this, the cicn, PixMap and CTable are now locked in memory right after
+   the first (and only) PlotCIcon call. To prevent fragmentation, the handles are
+   also moved to the top of the heap zone before this is done. */
+
+void
+mac_init_cicn(Image *img)
+{
+	CIconHandle cicnhandle;
+	MacImage *macimg;
+
+	macimg = (MacImage *) img->hook;
+	if (macimg->colricon != NULL && !macimg->cicn_inited) {
+		cicnhandle = (CIconHandle) macimg->colricon;
+
+		/* Hang 'em High! */
+		HLockHi((Handle) cicnhandle);
+		HLockHi((Handle) (*cicnhandle)->iconData);
+		HLockHi((Handle) (*cicnhandle)->iconPMap.pmTable);
+
+		/* Initialize PixMap pointer by this dummy call. */
+		PlotCIcon(NULL, cicnhandle);
+
+		/* Lock down stuff that got unlocked by PlotCIcon. */
+		HLock((Handle) (*cicnhandle)->iconData);
+		HLock((Handle) cicnhandle);
+		macimg->cicn_inited = TRUE;
+	}
+}
+
 /* Given a list of color table entries, create and return a color table. */
 
 static CTabHandle
@@ -743,15 +874,13 @@ make_generic_image_data(ImageFamily *imf)
 	MacImage *macimg;
 	Obj *colorcomp;
 
-	for (img = imf->images; img != NULL; img = img->next) {
+	for_all_images(imf, img) {
 		hasmono = hasmask = FALSE;
 		macimg = get_mac_image(img);
-		if (hasColorQD && macimg->colricon != nil) {
-			convert_cicn(img, (CIconHandle) macimg->colricon, &hasmono, &hasmask);
-		}
-		if (hasColorQD && macimg->colrpat != nil) {
-			convert_ppat(img, macimg->colrpat);
-		}
+		if (hasColorQD && macimg->colricon != nil)
+		  convert_cicn(img, (CIconHandle) macimg->colricon, &hasmono, &hasmask);
+		if (hasColorQD && macimg->colrpat != nil)
+		  convert_ppat(img, macimg->colrpat);
 		if (hasColorQD && img->w == 1 && img->h == 1 && macimg->colordefined) {
 			colorname = find_color_name(macimg->color.red, macimg->color.green, macimg->color.blue);
 			if (colorname != NULL) {
@@ -761,7 +890,6 @@ make_generic_image_data(ImageFamily *imf)
 								 cons(new_number(macimg->color.green),
 									  cons(new_number(macimg->color.blue),
 									       lispnil)));
-
 			}
 			img->palette = cons(cons(new_number(0), colorcomp), lispnil);
 		}
