@@ -54,8 +54,6 @@ static void select_player(int n);
 static int player_setup_dialog(void);
 static void set_player_setup_button_states(void);
 static pascal void draw_progress(WindowPtr win, short ditem);
-static void add_default_unit_image(ImageFamily *imf, int u, char *name);
-static void add_default_terrain_image(ImageFamily *imf, int t, char *name);
 static void add_default_emblem_image(ImageFamily *imf, int e, char *name);
 
 /* Global variables. */
@@ -2514,49 +2512,12 @@ init_unit_images()
 	uimages = (ImageFamily **) xmalloc(numutypes * sizeof(ImageFamily *));
 
 	for_all_unit_types(u) {
-		uimages[u] = get_unit_type_images(dside, u, (pref_color_unit_images ? "c" : "s"),
-										  mac_interp_imf, mac_load_imf,
-										  add_default_unit_image);
+		uimages[u] = get_unit_type_images(dside, u, (pref_color_unit_images ? "c" : "s"));
 	}
 	init_unit_cicns();
 }
 
-/* The default unit image is a box with the name of the unit inside. */
-
-static void
-add_default_unit_image(ImageFamily *imf, int u, char *name)
-{
-	Rect tmprect;
-	PicHandle pichandle;
-	RgnHandle tmprgn;
-	Image *img;
-	MacImage *macimg;
-
-	img = get_img(imf, 32, 32);
-    if (img == NULL)
-      return;
-	img->istile = FALSE;
-	macimg = get_mac_image(img);
-	/* (should define what grafport is in use) */
-	tmprgn = NewRgn();
-	GetClip(tmprgn);
-	SetRect(&tmprect, 0, 0, 32, 32);
-	ClipRect(&tmprect);
-	pichandle = OpenPicture(&tmprect);
-	InsetRect(&tmprect, 2, 2);
-	FillRect(&tmprect, QDPat(white));
-	InsetRect(&tmprect, 1, 1);
-	FrameRect(&tmprect);
-	MoveTo(4, 20);
-	DrawText(name, 0, strlen(name));
-	ClosePicture();
-	SetClip(tmprgn);
-	DisposeRgn(tmprgn);
-	macimg->monopict = pichandle;
-}
-
-/* Set up the collection of terrain images/patterns, making a default pattern
-   if necessary. */
+/* Set up the collection of terrain image patterns and colors. */
 
 void
 init_terrain_images()
@@ -2573,13 +2534,13 @@ init_terrain_images()
 	timages = (ImageFamily **) xmalloc(numttypes * sizeof(ImageFamily *));
 
 	for_all_terrain_types(t) {
-		timages[t] = get_terrain_type_images(dside, t, mac_interp_imf, mac_load_imf,
-											 add_default_terrain_image);
+		timages[t] = get_terrain_type_images(dside, t);
 	}
 	if (hasColorQD) {
 		tcolors = (MacImage **) xmalloc(numttypes * sizeof(MacImage *));
 		for_all_terrain_types(t) {
-			for (timg = timages[t]->images; timg != NULL; timg = timg->next) {
+			/* Look for an image that is really a solid color. */
+			for_all_images(timages[t], timg) {
 				mactimg = (MacImage *) timg->hook;
 				/* 1x1 images are actually colors. */
 				if (timg->w == 1 && timg->h == 1 && mactimg != NULL && mactimg->colordefined) {
@@ -2589,38 +2550,7 @@ init_terrain_images()
 			}
 		}
 	}
-	get_unseen_images(dside, mac_interp_imf, mac_load_imf, NULL);
-}
-
-/* Synthesize a terrain pattern, using the type to make a distinguishing
-   pattern. Basically the bits of the type number make a sort of bar code,
-   but enclosed in a box and with a horizontal bar through the middle, so
-   as to make it easier to tell which is which on a map.  (Trust me, I've
-   already experimented with this.) */
-
-static void
-add_default_terrain_image(ImageFamily *imf, int t, char *name)
-{
-	Image *img;
-	MacImage *macimg;
-
-	imf->ersatz = TRUE;
-	img = get_img(imf, 8, 8);
-	if (img == NULL)
-	  return;
-	img->minw = img->minh = 1;
-	img->maxw = img->maxh = 9999;
-	img->istile = TRUE;
-	macimg = get_mac_image(img);
-	SET_IMG_PAT (macimg, 0, 0x01);
-	SET_IMG_PAT (macimg, 1, (t << 2) + 0x01);
-	SET_IMG_PAT (macimg, 2, (t << 2) + 0x01);
-	SET_IMG_PAT (macimg, 3, 0x7d);
-	SET_IMG_PAT (macimg, 4, (t << 2) + 0x01);
-	SET_IMG_PAT (macimg, 5, (t << 2) + 0x01);
-	SET_IMG_PAT (macimg, 6, 0x01);
-	SET_IMG_PAT (macimg, 7, 0xff);
-	macimg->patdefined = 1;
+	get_unseen_images(dside);
 }
 
 /* Given a color name, get actual RGB values for it.  As a special case for b/w
@@ -2664,6 +2594,8 @@ init_all_emblem_images()
 {
 	Side *side2;
 
+	/* Allocate enough space to account for both initial sides, and for ones that
+       may be added later. */
 	eimages = (ImageFamily **) xmalloc((g_sides_max() + 1) * sizeof(ImageFamily *));
 
 	for_all_sides_plus_indep(side2) {
@@ -2674,9 +2606,6 @@ init_all_emblem_images()
 void
 reinit_all_emblem_images()
 {
-	Side *side2;
-	int i;
-
 	dispose_emblem_cicns();
 	/* This isn't great, because all the collected data goes away, and we have to
 	   reread and reallocate everything.  It would be smarter just to reinit the
@@ -2691,69 +2620,99 @@ reinit_all_emblem_images()
 void
 init_emblem_images(Side *side2)
 {
-	int e;
+	int s2 = side_number(side2);
 
-	e = side_number(side2);
-	eimages[e] = get_emblem_images(dside, side2, mac_interp_imf, mac_load_imf,
-								   add_default_emblem_image);
+	eimages[s2] = get_emblem_images(dside, side2);
 }
 
-/* Default image is SICN-sized number.  Use only if no images have been found
-   and the emblem name is not "none". */
+/* The purpose of these 4 functions is to ensure the current cicn set is properly
+   initialized through a dummy PlotCIcon call.  */
 
-static void
-add_default_emblem_image(ImageFamily *imf, int e, char *name)
+/* Init the unit cicns and lock them at the top of the heap zone. */
+
+void mac_init_cicn(Image *img);
+
+void
+init_unit_cicns()
 {
-	int i;
-	char tmpbuf[10];
-	Str255 tmpstr;
-	Handle sicnhandle, maskhandle;
-	Rect tmprect;
-	PicHandle pichandle;
-	RgnHandle tmprgn;
+	int u;
 	Image *img;
-	MacImage *macimg;
-
-	img = get_img(imf, 16, 16);
-	img->minw = img->minh = 8;
-	img->maxw = img->maxh = 64;
-	/* Emblems never tile. */
-	img->istile = FALSE;
-	macimg = get_mac_image(img);
-	sprintf(tmpbuf, "s%d", e);
-	c2p(name, tmpstr);
-	sicnhandle = GetNamedResource('SICN', tmpstr);
-	if (tmpstr != nil) {
-		/* Found a sicn with the side's number in it. */
-		/* Image itself is the first 32 bytes, mask is second 32 if present. */
-		macimg->monosicn = sicnhandle;
-		maskhandle = (Handle) NewHandle(32);
-		if (SizeResource(sicnhandle) >= 64) {
-			for (i = 0; i < 32; ++i) {
-				(*maskhandle)[i] = (*sicnhandle)[i+32];
-			}
-		} else {
-			/* Set up an all-white background. */
-			for (i = 0; i < 32; ++i) {
-				(*maskhandle)[i] = 0xff;
+	
+	for_all_unit_types(u) {
+		for_all_images(uimages[u], img) {
+			if (img && img->hook) {
+				mac_init_cicn(img);
 			}
 		}
-		macimg->masksicn = maskhandle;
-	} else {
-		/* If we couldn't even find a number sicn, build a crude substitute pict. */
-		imf->ersatz = TRUE;
-		tmprgn = NewRgn();
-		GetClip(tmprgn);
-		SetRect(&tmprect, 0, 0, 16, 16);
-		ClipRect(&tmprect);
-		pichandle = OpenPicture(&tmprect);
-		FillRect(&tmprect, QDPat(white));
-		MoveTo(2, 12);
-		DrawText(tmpbuf+1, 0, strlen(tmpbuf+1));
-		ClosePicture();
-		SetClip(tmprgn);
-		DisposeRgn(tmprgn);
-		macimg->monopict = pichandle;
 	}
 }
 
+/* Trash the unit cicns. */
+
+void
+dispose_unit_cicns()
+{
+	int u;
+	CIconHandle cicnhandle;
+	MacImage *macimg;
+	Image *img;
+	
+	for_all_unit_types(u) {
+		for_all_images(uimages[u], img) {
+			if (img && img->hook) {
+				macimg = (MacImage *) img->hook;
+				if (macimg->colricon != NULL) {
+					cicnhandle = (CIconHandle) macimg->colricon;
+					DisposHandle((Handle) (*cicnhandle)->iconPMap.pmTable);
+					DisposHandle((Handle) (*cicnhandle)->iconData);
+					DisposHandle((Handle) cicnhandle);
+					macimg->colricon = nil;
+					macimg->cicn_inited = FALSE;
+				}
+			}
+		}
+	}
+}
+
+/* Init the emblem cicns and lock them at the top of the heap zone. */
+
+void
+init_emblem_cicns()
+{
+	Side *side;
+	Image *img;
+	
+	for_all_sides_plus_indep(side) {
+		for_all_images(eimages[side_number(side)], img) {
+			if (img && img->hook) {
+				mac_init_cicn(img);
+			}
+		}
+	}
+}
+
+/* Trash the emblem cicns, thus clearing up the high part of the heap. */
+
+void
+dispose_emblem_cicns()
+{
+	Side *side;
+	CIconHandle cicnhandle;
+	MacImage *macimg;
+	Image *img;
+	return;
+	for_all_sides_plus_indep(side) {
+		for (img = eimages[side_number(side)]->images; img != NULL; img = img->next) {
+			if (img && img->hook) {
+				macimg = (MacImage *) img->hook;
+				if (macimg->colricon != NULL) {
+					cicnhandle = (CIconHandle) macimg->colricon;
+					DisposHandle((Handle) (*cicnhandle)->iconPMap.pmTable);
+					DisposHandle((Handle) (*cicnhandle)->iconData);
+					DisposHandle((Handle) cicnhandle);
+					macimg->colricon = nil;
+				}
+			}
+		}
+	}
+}
