@@ -13,6 +13,8 @@ extern void xform_fractional(Map *map, int x, int y, int xf, int yf, int *sxp, i
 
 extern int contour_interval;
 
+#define GENERIC_MERIDIANS
+
 static void draw_meridians(Map *map);
 static void draw_borders(Map *map, int x, int y, int b);
 static void draw_connections(Map *map, int x, int y, int c);
@@ -1481,6 +1483,8 @@ draw_other_map(Map *map, Map *map2)
 	PenNormal();
 }
 
+#ifdef GENERIC_MERIDIANS
+
 static void meridian_line_callback(int x1, int y1, int x1f, int y1f, int x2, int y2, int x2f, int y2f);
 static void meridian_text_callback(int x1, int y1, int x1f, int y1f, char *str);
 
@@ -1528,6 +1532,176 @@ meridian_text_callback(int x1, int y1, int x1f, int y1f, char *str)
 	draw_legend_text(sx1, sy1 - 12, 12, tmpbuf, 0, 0, 0);
 }
 
+#else
+
+void
+draw_meridians(Map *map)
+{
+	int latmin, latcen, latmax, lonmin, loncen, lonmax, lat, lon; 
+	int xfmin, xfmax, yfmin, yfmax, xmin, ymin, xmax, ymax;
+	int right, left, top, bottom, xend, ymid, incr, interval;
+    	int latdeg, latmit, londeg, lonmit;
+	int x, y, xf, yf, sx, sx1, sy;
+	Rect portrect, drawrect;
+    	char mitbuf[4];
+
+	/* Set origin to Greenwich x equator in wrapped worlds. */
+	if (area.xwrap) {
+		loncen = latcen = 0;
+	/* Else set origin to area center. */
+	} else {
+		loncen = 60 * area.longitude;
+		latcen = 60 * area.latitude;
+	}
+
+	/* Compute the interval in pixels. */
+    	incr = map->vp->latlong_interval;
+    	latlong_to_xy(latcen, loncen, &xmin, 0, &xfmin, 0);
+    	latlong_to_xy(latcen, loncen + incr, &xmax, 0, &xfmax, 0);
+	/* Skip if next meridian is outside the area. */
+	if (!in_area(xmax, 0)) return;
+	xform_fractional(map, xmin, 0, xfmin, 0, &sx, 0);
+	xform_fractional(map, xmax, 0, xfmax, 0, &sx1, 0);
+	interval = abs(sx1 - sx);
+	/* Skip if interval is too small. */
+	if (interval < 16) return;
+		
+	/* Set up the drawing rect. */
+	drawrect = portrect = map->gworldPortPtr->portRect;
+	drawrect.right = min(portrect.right, map->vp->totsw);
+	drawrect.bottom = min(portrect.bottom, map->vp->totsh);
+	/* A hack for better positioning of the text. */	
+	drawrect.top += map->vp->hh/4; 
+	drawrect.bottom -= map->vp->hh/8; 
+	
+	/* Compute min/max values for latitudes and longitudes. */
+	nearest_cell(map->vp, -map->conw, drawrect.bottom - map->toph, 
+			    &xmin, &ymin, &xfmin, &yfmin);
+	nearest_cell(map->vp, drawrect.right - map->conw, -map->toph, 
+			    &xmax, &ymax, &xfmax, &yfmax);
+	xy_to_latlong(xmin, ymin, xfmin, yfmin, &latmin, &lonmin);
+	xy_to_latlong(xmax, ymax, xfmax, yfmax, &latmax, &lonmax);
+
+	/* Center the grid on the origin. */
+	lonmin += (loncen - lonmin) % incr;
+	lonmax += (loncen - lonmax) % incr;
+	latmin += (latcen - latmin) % incr;
+	latmax += (latcen - latmax) % incr;
+	/* Special fixes for wrapped gworlds etc.*/
+	if (lonmin > 0 || lonmax < 0 || lonmax <= lonmin) {
+		lonmin = -10800; 
+		lonmax = 10800 - incr;
+	}
+	
+	/* Used in clipping code. */
+	ymid = area.halfheight;
+	xend = area.width - 1;
+
+	/* Set meridian color. */
+	RGBForeColor(&meridiancolor);
+	
+	/* Draw the latitiudes. */
+	for (lat = latmin; lat <= latmax; lat += incr) {
+		left = drawrect.left; 
+		right = drawrect.right;
+		latlong_to_xy(lat, loncen, &x, &y, &xf, &yf);
+		/* Skip if outside area. */
+		if (!in_area(x, y)) continue;
+		xform_fractional(map, x, y, xf, yf, &sx, &sy);
+		/* Don't draw latitudes where the longitude text will be. */
+		if (sy - drawrect.top < 16 || drawrect.bottom - sy < 16) continue;
+		/* Clip corners to area for non-wrapped worlds. */
+		if (!area.xwrap) {
+			/* Compute left endpoint. */
+			if (in_area(0,y)) { 							
+				 xform(map, 0, y, &left, 0);
+				 left += 32;
+			} else if (in_area(ymid - y, y)) {					
+				 xform(map, ymid - y, y, &left, 0);
+				 left += 32;
+			}
+			/* Compute right endpoint. */
+			if (in_area(xend + ymid - y, y)) {				
+				 xform(map, xend + ymid - y, y, &right, 0);
+				 right -= 32 - map->vp->hw;
+			} else if (in_area(xend, y)) {					
+				 xform(map, xend, y, &right, 0);
+				 right -= 32 - map->vp->hw;
+			}
+			/* Make room for text at left end. */
+			left = max(left, portrect.left + 32);
+			/* Make room for text at right end. */
+			right = min(right, portrect.right - 32);
+		} else { 
+			/* Make room for text at right end unless the gworld is wrapped. */
+			if (map->vp->sxmax > portrect.right - portrect.left) right -= 32;
+			/* Always make room at left end. */
+			left += 32; 
+		}
+		MoveTo(left, sy);
+		LineTo(right, sy);
+		latdeg = abs(lat) / 60;
+		latmit = abs(lat) % 60;
+		mitbuf[0] = '\0';
+		if (latmit != 0) sprintf(mitbuf, "%d", latmit);
+		sprintf(tmpbuf, "%d%s", latdeg, mitbuf);
+		if (lat >= 0) strcat(tmpbuf, "N");
+		if (lat < 0) strcat(tmpbuf, "S");
+		/* Always draw latitude text at left edge. */
+		draw_legend_text(left - 3, sy, 12, tmpbuf, +1, 0, 0);
+		/* Also draw text at right edge unless the gworld is wrapped. */
+		if (!area.xwrap || map->vp->sxmax > portrect.right - portrect.left)
+			draw_legend_text(right + 3, sy, 12, tmpbuf, -1, 0, 0);
+	}
+
+	/* Draw the longitudes. */
+	for (lon = lonmin; lon <= lonmax; lon += incr) {
+		/* Make room for top and bottom text. */
+		top = drawrect.top + 16; 
+		bottom = drawrect.bottom - 16;
+		latlong_to_xy(latcen, lon, &x, &y, &xf, &yf);
+		/* Skip if outside area. */
+		if (!in_area(x, y)) continue;
+		xform_fractional(map, x, y, xf, yf, &sx, &sy);
+		/* Clip corners to area for non-wrapped maps. */
+		if (!area.xwrap) {
+			/* Compute top endpoint. */
+			if (in_area(0,  ymid + 2 * x))						
+				 xform(map, 0, ymid + 2 * x, 0, &top);
+			else if (in_area(2 * x - xend, ymid + 2 * (xend - x)))	
+				 xform(map, 2 * x - xend, ymid + 2 * (xend - x), 0, &top);
+			/* Compute bottom endpoint. */
+			if (in_area(2 * x, ymid - 2 * x))					
+				 xform(map, 2 * x, ymid - 2 * x, 0, &bottom);	
+			else if (in_area(xend, ymid - 2 * (xend - x)))				
+				 xform(map, xend, ymid - 2 * (xend - x), 0, &bottom);
+		}
+		/* Wrap sx if necessary. */
+		if (area.xwrap && sx < 0)
+			sx += map->vp->sxmax;
+		else if (area.xwrap && sx > map->vp->sxmax)
+			sx -= map->vp->sxmax;
+		MoveTo(sx, top);
+		LineTo(sx, bottom);
+		londeg = abs(lon) / 60;
+		lonmit = abs(lon) % 60;
+		mitbuf[0] = '\0';
+		if (lonmit != 0) sprintf(mitbuf, "%d", lonmit);
+		sprintf(tmpbuf, "%d%s", londeg, mitbuf);
+		if (lon > 0) strcat(tmpbuf, "E");
+		if (lon < 0) strcat(tmpbuf, "W");
+		/* Draw text at top and bottom. */
+		if (top == drawrect.top + 16)
+			draw_legend_text(sx, top - 6, 12, tmpbuf, 0, 0, 0);
+		if (bottom == drawrect.bottom - 16)
+			draw_legend_text(sx, bottom + 6, 12, tmpbuf, 0, 0, 0);
+	}
+
+	ForeColor(blackColor);
+}
+
+#endif
+
 /* x0 may be negative here. */
 
 /* (should add a "draw interior only" version of this routine?) */
