@@ -1,5 +1,5 @@
 /* Handling of assorted minor windows for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,9 +7,15 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int construction_possible PARAMS ((int u2));
-extern int can_build_or_help PARAMS ((Unit *unit));
-extern char *side_score_desc PARAMS ((char *buf, Side *side, Scorekeeper *sk));
+extern char *curdatestr;
+extern char *goal_desc PARAMS ((char *buf, Goal *goal));
+extern int update_total_hist_lines PARAMS ((Side *side));
+extern int build_hist_contents PARAMS ((Side *side, int n, HistEvent **histcontents, int numvis));
+
+#define unit_doctrine(unit)  \
+  ((unit->side ? unit->side : indepside)->udoctrine[unit->type])
+
+#include "kpublic.h"
 #include "macconq.h"
 extern void set_construction_run_length(int len);
 
@@ -21,6 +27,9 @@ static pascal void history_scroll_fn(ControlHandle control, short code);
 static pascal void notice_vscroll_fn(ControlHandle control, short code);
 static pascal void scores_vscroll_fn(ControlHandle control, short code);
 
+static void calc_history_layout(void);
+static void set_history_scrollbar(void);
+
 /* Globals for the game window. */
 
 WindowPtr gamewin = nil;
@@ -55,6 +64,7 @@ WindowPtr constructionwin = nil;
 static ListHandle construction_unit_list = nil;
 static ListHandle construction_type_list = nil;
 
+static int maxtypewid;
 static int constructmargin = 5;
 static int constructtop = 32;
 
@@ -87,7 +97,7 @@ static int m_per_row = 2;
 
 static char *nummrows;
 
-int closeupspacing = 16;
+int closeup_spacing = 16;
 
 int closeupwinwid = 200;
 
@@ -95,19 +105,17 @@ int closeupwinwid = 200;
 
 WindowPtr historywin = nil;
 
+int maxvishistlines = 200;
+
 HistEvent **histcontents = NULL;
 
 int numhistcontents = 0;
 
 ControlHandle histvscrollbar;
 
-int history_line_spacing = 15; /* (should derive from font) */
+int history_line_spacing;
 
-int history_top_line_height = 25;
-
-HistEvent *firstvisevt = NULL;
-
-HistEvent *secondvisevt = NULL;
+int history_top_line_height;
 
 int numvishistlines;
 
@@ -223,6 +231,8 @@ draw_game()
 	}
 	GetPort(&oldport);
 	SetPort(gamewin);
+	TextFont(large_font_id);
+	TextSize(large_font_size);
 	draw_game_date();
 	draw_game_progress();
 	/* Draw a solid separating line between date info and side list. */
@@ -244,9 +254,8 @@ draw_game_date()
 	EraseRect(&tmprect);
 	MoveTo(tmprect.left + 10, tmprect.top + 12);
 	TextFace(bold);
-	DrawString((unsigned char *) curdatestr);
+	DrawText(curdatestr, 0, strlen(curdatestr));
 	TextFace(0);
-	/* (should draw season name here somewhere?) */
 	draw_game_clocks();
 	if (endofgame) {
 		gray_out_rect(&tmprect);
@@ -545,13 +554,12 @@ create_construction_window()
 	constructbutton = GetNewControl(cConstructButton, constructionwin);
 	researchbutton = GetNewControl(cResearchButton, constructionwin);
 	SetPort(constructionwin);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	calc_construction_rects();
 	run_length_text = TENew(&runlengthrect, &runlengthrect);
 	set_construction_run_length(DEFAULT_RUN);
 	editedrunlength = -1;
-	/* Switch to a font for the lists. */
-	TextFont(monaco);
-	TextSize(9);
 	/* Set up the list of all constructing units. */
 	tmprect = unitlistrect;
 	tmprect.right -= sbarwid;
@@ -696,14 +704,16 @@ calc_construction_rects()
 	int wid, hgt, divide;
 	Rect tmprect;
 
+	/* (should compute based on num chars needed, plus font size) */
+	maxtypewid = 250;
 	tmprect = constructionwin->portRect;
 	runlengthrect = tmprect;
 	runlengthrect.left = runlengthrect.right - 100;  runlengthrect.top = 5;
 	runlengthrect.right -= 20;  runlengthrect.bottom = 25;
 	wid = tmprect.right - tmprect.left - sbarwid;
 	hgt = tmprect.bottom - tmprect.top - sbarwid;
-	if (wid / 2 > 220 /* maxtypewid */) {
-		divide = wid - 220;
+	if (wid / 2 > maxtypewid) {
+		divide = wid - maxtypewid;
 	} else {
 		divide = wid / 2;
 	}
@@ -734,7 +744,7 @@ get_selected_construction_unit()
 	if (LGetSelect(TRUE, &tmpcell, construction_unit_list)) {				
 		if (tmpcell.v < constructor_units->numunits) {
 			unit = (constructor_units->units)[tmpcell.v].unit;
-			if (is_acting(unit))
+			if (is_active(unit))
 			  return unit;
 		}
 	}
@@ -767,7 +777,7 @@ scroll_to_selected_construction_unit()
 	}
 	unit = get_selected_construction_unit();
 	if (unit != NULL && inside_area(unit->x, unit->y))
-	  scroll_best_map_to_unit(unit);
+	  scroll_best_map_to_unit(unit, FALSE);
 }
 
 /* Handle a click anywhere within the construction window. */
@@ -788,11 +798,12 @@ do_mouse_down_construction(Point mouse, int mods)
 		if (unit != NULL) {
 			u = get_selected_construction_type();
 			if (u != NONUTYPE) {
-				push_build_task(unit, u, currunlength);
-				execute_task(unit);
+				net_push_build_task(unit, u, currunlength);
 				update_construction_unit_list(unit);
-				if (modal_construction && window_behind_construction != nil)
-				  SelectWindow(window_behind_construction);
+				if (modal_construction && window_behind_construction != nil) {
+					SelectWindow(window_behind_construction);
+					update_window(window_behind_construction);
+				}
 				window_behind_construction = NULL;
 				modal_construction = FALSE;
 				return;
@@ -803,11 +814,12 @@ do_mouse_down_construction(Point mouse, int mods)
 		if (unit != NULL) {
 			u = get_selected_construction_type();
 			if (u != NONUTYPE) {
-				push_research_task(unit, u, u_tech_to_build(u));
-				execute_task(unit);
+				net_push_research_task(unit, u, u_tech_to_build(u));
 				update_construction_unit_list(unit);
-				if (modal_construction && window_behind_construction != nil)
-				  SelectWindow(window_behind_construction);
+				if (modal_construction && window_behind_construction != nil) {
+					SelectWindow(window_behind_construction);
+					update_window(window_behind_construction);
+				}
 				window_behind_construction = NULL;
 				modal_construction = FALSE;
 				return;
@@ -944,7 +956,7 @@ maybe_add_unit_to_construction_list(Unit *unit)
 {
 	Point tmpcell;
 
-	if (is_acting(unit)
+	if (is_active(unit)
 		&& can_build_or_help(unit)
 		&& side_controls_unit(dside, unit)) {
 		/* Add this unit to the vector of constructing units. */
@@ -969,7 +981,7 @@ update_unit_list_for_type(int u)
 		unit = (constructor_units->units)[i].unit;
 		if (unit != NULL) {
 			SetPt(&tmpcell, 0, i);
-			if (is_acting(unit) && unit->side == dside) {
+			if (is_active(unit) && unit->side == dside) {
 				construction_desc(spbuf, unit, u);
 				LSetCell(spbuf, strlen(spbuf), tmpcell, construction_unit_list);
 			} else {
@@ -1032,10 +1044,8 @@ adjust_construction_controls()
 	   run length in the dialog. */
 	if (editedrunlength < 0 && unit != NULL && u != NONUTYPE) {
 		len = DEFAULT_RUN;
-		if (dside->udoctrine[unit->type] != NULL
-			&& dside->udoctrine[unit->type]->construction_run != NULL
-			&& dside->udoctrine[unit->type]->construction_run[u] > 0) {
-			len = (dside->udoctrine[unit->type])->construction_run[u];
+		if (unit_doctrine(unit)->construction_run[u] > 0) {
+			len = unit_doctrine(unit)->construction_run[u];
 		}
 		set_construction_run_length(len);
 	}
@@ -1096,6 +1106,9 @@ zoom_construction(int part)
 static void
 adjust_construction_items()
 {
+	int listwid;
+	Point cellsize;
+
 	/* Recalculate size and position. */
 	calc_construction_rects();
 	/* Resize the run length text item. */
@@ -1103,55 +1116,45 @@ adjust_construction_items()
 	(*run_length_text)->destRect = runlengthrect;
 	TECalText(run_length_text);
 	/* Resize the unit list. */
-	LSize(unitlistrect.right - unitlistrect.left - sbarwid,
-		  unitlistrect.bottom - unitlistrect.top,
+	listwid = unitlistrect.right - unitlistrect.left - sbarwid;
+	LSize(listwid, unitlistrect.bottom - unitlistrect.top,
 		  construction_unit_list);
+	SetPt(&cellsize, max(300, listwid), 12);
+	LCellSize(cellsize, construction_unit_list);
 	/* Move the type list (is this the approved way to do it?) */
 	(*construction_type_list)->rView.left = typelistrect.left;
-	LSize(typelistrect.right - typelistrect.left - sbarwid,
-		  typelistrect.bottom - typelistrect.top,
+	listwid = typelistrect.right - typelistrect.left - sbarwid;
+	LSize(listwid, typelistrect.bottom - typelistrect.top,
 		  construction_type_list);
+	SetPt(&cellsize, max(300, listwid), 12);
+	LCellSize(cellsize, construction_type_list);
 }
 
 /* The side renaming dialog includes places for all the different name-related
    properties of a side. */
 
+static void seed_side_rename_string(DialogPtr win, short ditem, char *str);
+static char *get_side_rename_string(DialogPtr win, short ditem);
+
 void
 side_rename_dialog(Side *side)
 {
 	short done = FALSE, changed = TRUE, ditem;
-	Str255 tmpstr;
+	char *str;
 	DialogPtr win;
-	short itemtype;  Handle itemhandle;  Rect itemrect;
 
 	win = GetNewDialog(dSideRename, NULL, (DialogPtr) -1L);
 	while (!done) {
 		if (changed) {
 			/* Seed the items with the current side names. */
-			GetDItem(win, diSideRenameName, &itemtype, &itemhandle, &itemrect);
-			c2p((side->name ? side->name : ""), tmpstr);
-			SetIText(itemhandle, tmpstr);
-			GetDItem(win, diSideRenameFullName, &itemtype, &itemhandle, &itemrect);
-			c2p((side->longname ? side->longname : ""), tmpstr);
-			SetIText(itemhandle, tmpstr);
-			GetDItem(win, diSideRenameAcronym, &itemtype, &itemhandle, &itemrect);
-			c2p((side->shortname ? side->shortname : ""), tmpstr);
-			SetIText(itemhandle, tmpstr);
-			GetDItem(win, diSideRenameNoun, &itemtype, &itemhandle, &itemrect);
-			c2p((side->noun ? side->noun : ""), tmpstr);
-			SetIText(itemhandle, tmpstr);
-			GetDItem(win, diSideRenamePluralNoun, &itemtype, &itemhandle, &itemrect);
-			c2p((side->pluralnoun ? side->pluralnoun : ""), tmpstr);
-			SetIText(itemhandle, tmpstr);
-			GetDItem(win, diSideRenameAdjective, &itemtype, &itemhandle, &itemrect);
-			c2p((side->adjective ? side->adjective : ""), tmpstr);
-			SetIText(itemhandle, tmpstr);
-			GetDItem(win, diSideRenameEmblemName, &itemtype, &itemhandle, &itemrect);
-			c2p((side->emblemname ? side->emblemname : ""), tmpstr);
-			SetIText(itemhandle, tmpstr);
-			GetDItem(win, diSideRenameColorScheme, &itemtype, &itemhandle, &itemrect);
-			c2p((side->colorscheme ? side->colorscheme : ""), tmpstr);
-			SetIText(itemhandle, tmpstr);
+			seed_side_rename_string(win, diSideRenameName, side->name);
+			seed_side_rename_string(win, diSideRenameFullName, side->longname);
+			seed_side_rename_string(win, diSideRenameAcronym, side->shortname);
+			seed_side_rename_string(win, diSideRenameNoun, side->noun);
+			seed_side_rename_string(win, diSideRenamePluralNoun, side->pluralnoun);
+			seed_side_rename_string(win, diSideRenameAdjective, side->adjective);
+			seed_side_rename_string(win, diSideRenameEmblemName, side->emblemname);
+			seed_side_rename_string(win, diSideRenameColorScheme, side->colorscheme);
 			ShowWindow(win);
 			changed = FALSE;
 		}
@@ -1161,23 +1164,23 @@ side_rename_dialog(Side *side)
 		switch (ditem) {
 			case diSideRenameOK:
 				/* Actually change the side's slots. */
-				/* (should all go through networkable calls!) */
-				GetDItem(win, diSideRenameName, &itemtype, &itemhandle, &itemrect);
-				set_side_name(dside, dside, get_string_from_item(itemhandle));
-				GetDItem(win, diSideRenameFullName, &itemtype, &itemhandle, &itemrect);
-				side->longname = get_string_from_item(itemhandle);
-				GetDItem(win, diSideRenameAcronym, &itemtype, &itemhandle, &itemrect);
-				side->shortname = get_string_from_item(itemhandle);
-				GetDItem(win, diSideRenameNoun, &itemtype, &itemhandle, &itemrect);
-				side->noun = get_string_from_item(itemhandle);
-				GetDItem(win, diSideRenamePluralNoun, &itemtype, &itemhandle, &itemrect);
-				side->pluralnoun = get_string_from_item(itemhandle);
-				GetDItem(win, diSideRenameAdjective, &itemtype, &itemhandle, &itemrect);
-				side->adjective = get_string_from_item(itemhandle);
-				GetDItem(win, diSideRenameEmblemName, &itemtype, &itemhandle, &itemrect);
-				side->emblemname = get_string_from_item(itemhandle);
-				GetDItem(win, diSideRenameColorScheme, &itemtype, &itemhandle, &itemrect);
-				side->colorscheme = get_string_from_item(itemhandle);
+				net_set_side_name(dside, side, get_side_rename_string(win, diSideRenameName));
+				net_set_side_longname(dside, side, get_side_rename_string(win, diSideRenameFullName));
+				net_set_side_shortname(dside, side, get_side_rename_string(win, diSideRenameAcronym));
+				net_set_side_noun(dside, side, get_side_rename_string(win, diSideRenameNoun));
+				net_set_side_pluralnoun(dside, side, get_side_rename_string(win, diSideRenamePluralNoun));
+				net_set_side_adjective(dside, side, get_side_rename_string(win, diSideRenameAdjective));
+				net_set_side_emblemname(dside, side, get_side_rename_string(win, diSideRenameEmblemName));
+				net_set_side_colorscheme(dside, side, get_side_rename_string(win, diSideRenameColorScheme));
+
+				/* Trash and reload all emblem cicns at the top of the heap zone,
+				   so as to avoid fragmentation */				
+				reinit_all_emblem_images();
+
+				dissect_colorscheme(side_number(side), side->colorscheme);
+				ui_update_state();
+				save_preferences();
+				
 				/* Tweak the side menu. */
 				update_side_menu(dside);
 				/* Force redisplay of everything that might use any side names. */
@@ -1187,18 +1190,43 @@ side_rename_dialog(Side *side)
 				done = TRUE;
 				break;
 			case diSideRenameRandom:
-				side->name = NULL;
-				side->noun = NULL;
-				/* always need to clear this cache before renaming... */
-				side->pluralnoun = NULL;
-				side->adjective = NULL;
+				net_set_side_name(dside, side, NULL);
+				net_set_side_noun(dside, side, NULL);
+				/* Always need to clear the plural noun cache before renaming... */
+				net_set_side_pluralnoun(dside, side, NULL);
+				net_set_side_adjective(dside, side, NULL);
 				make_up_side_name(side);
 				init_emblem_images(side);
 				changed = TRUE;
 				break;
 		}
 	}
-	DisposDialog(win);
+	DisposeDialog(win);
+	update_all_map_windows();
+}
+
+static void
+seed_side_rename_string(DialogPtr win, short ditem, char *str)
+{
+	Str255 tmpstr;
+	short itemtype;  Handle itemhandle;  Rect itemrect;
+
+	GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
+	c2p((str ? str : ""), tmpstr);
+	SetIText(itemhandle, tmpstr);
+}
+
+static char *
+get_side_rename_string(DialogPtr win, short ditem)
+{
+	char *str;
+	short itemtype;  Handle itemhandle;  Rect itemrect;
+
+	GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
+	str = get_string_from_item(itemhandle);
+	if (empty_string(str))
+	  str = NULL;
+	return str;
 }
 
 /* Unit naming/renaming. */
@@ -1234,7 +1262,7 @@ unit_rename_dialog(Unit *unit)
 		switch (ditem) {
 			case diRenameOK:
 				GetDItem(win, diRenameName, &itemtype, &itemhandle, &itemrect);
-				set_unit_name(dside, unit, get_string_from_item(itemhandle));
+				net_set_unit_name(dside, unit, get_string_from_item(itemhandle));
 				/* Fall into the next case. */
 			case diRenameCancel:
 				done = TRUE;
@@ -1249,7 +1277,7 @@ unit_rename_dialog(Unit *unit)
 				break;
 		}
 	}
-	DisposDialog(win);
+	DisposeDialog(win);
 	return TRUE;
 }
 
@@ -1306,7 +1334,8 @@ preferred_closeup_size(int u, int *widp, int *hgtp)
 {
 	int wid = closeupwinwid, hgt, m, u2, count;
 
-	hgt = 4 + 32 + 4 + 3 * closeupspacing;
+	closeup_spacing = small_line_spacing + 1;
+	hgt = 4 + 32 + 4 + 3 * closeup_spacing;
 	if (nummrows == NULL) {
 		/* Compute and cache the space needed to display each unit's supply. */
 		nummrows = xmalloc(numutypes);
@@ -1321,9 +1350,9 @@ preferred_closeup_size(int u, int *widp, int *hgtp)
 		  	}
 		}
 	}
-	hgt += nummrows[u] * closeupspacing;
+	hgt += nummrows[u] * closeup_spacing;
 	if (u_acp(u) > 0)
-	  hgt += 5 * closeupspacing;
+	  hgt += 5 * closeup_spacing;
 	/* (should make some room for tooling state) */
 	*widp = wid;  *hgtp = hgt;
 }
@@ -1333,8 +1362,8 @@ preferred_closeup_size(int u, int *widp, int *hgtp)
 void
 draw_unit_closeup(UnitCloseup *unitcloseup)
 {
-	int u, m, sx = 4, sy, count, sx2, sy2;
-	char tmpbuf[BUFSIZE];
+	int u, m, sx = 4, sy, count, mrow, sx2, sy2;
+	char infobuf[BUFSIZE];
 	Rect tmprect;
 	GrafPtr oldport;
 	WindowPtr win = unitcloseup->window;
@@ -1351,9 +1380,9 @@ draw_unit_closeup(UnitCloseup *unitcloseup)
 	}
 	GetPort(&oldport);
 	SetPort(win);
-	/* Switch to a small fixed-width font. */
-	TextFont(monaco);
-	TextSize(9);
+	/* Use the small font. */
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	EraseRect(&win->portRect);
 	u = unit->type;
 	/* Draw the unit's image. */
@@ -1361,109 +1390,73 @@ draw_unit_closeup(UnitCloseup *unitcloseup)
 	EraseRect(&tmprect);
 	draw_unit_image(win, tmprect.left, tmprect.top,
 					tmprect.right - tmprect.left, tmprect.bottom - tmprect.top,
-					u, side_number(unit->side), !completed(unit));
+					u, side_number(unit->side), !completed(unit), TRUE);
 	/* Draw the unit's name. */
-	name_or_number(unit, tmpbuf);
-	MoveTo(44, 4 + closeupspacing);
-	DrawText(tmpbuf, 0, strlen(tmpbuf));
+	name_or_number(unit, infobuf);
+	MoveTo(44, 4 + closeup_spacing);
+	DrawText(infobuf, 0, strlen(infobuf));
 	if (Debug || DebugG || DebugM) {
-		sprintf(tmpbuf, " %d ", unit->id);
+		sprintf(infobuf, " %d ", unit->id);
 		MoveTo(closeupwinwid - 30, 15);
-		DrawText(tmpbuf, 0, strlen(tmpbuf));
+		DrawText(infobuf, 0, strlen(infobuf));
 	}
 	/* Draw the unit's side and type. */
-	side_and_type_name(tmpbuf, dside, u, unit->side);
-	MoveTo(44, 4 + closeupspacing * 2);
-	DrawText(tmpbuf, 0, strlen(tmpbuf));
+	side_and_type_name(infobuf, dside, u, unit->side);
+	MoveTo(44, 4 + closeup_spacing * 2);
+	DrawText(infobuf, 0, strlen(infobuf));
 	/* Draw the unit's location. */
-	if (unit->z != 0) {
-		sprintf(tmpbuf, "; alt %d", unit->z);
-	} else {
-		strcpy(tmpbuf, "");
-	}
-	sprintf(spbuf, "at %d,%d%s", unit->x, unit->y, tmpbuf);
-	if (unit->transport != NULL) {
-		sprintf(tmpbuf, "In %s (%s)", short_unit_handle(unit->transport), spbuf);
-	} else {
-		strcpy(tmpbuf, spbuf);
-	}
-	sy = 4 + 32 + 4 + closeupspacing;
+	location_desc(infobuf, dside, unit, unit->type, unit->x, unit->y);
+	sy = 4 + 32 + 4 + closeup_spacing;
 	MoveTo(sx, sy);
-	DrawText(tmpbuf, 0, strlen(tmpbuf));
+	DrawText(infobuf, 0, strlen(infobuf));
+	/* Very briefly list the numbers and types of the occupants. */
+	occupants_desc(infobuf, unit);
+	sy += closeup_spacing;
+	MoveTo(sx, sy);
+	DrawText(infobuf, 0, strlen(infobuf));
 	/* Draw the unit's hit points. */
-	hp_desc(tmpbuf, unit, TRUE);
-	sy += closeupspacing;
+	hp_desc(infobuf, unit, TRUE);
+	strcat(infobuf, "   ");
+	acp_desc(infobuf + strlen(infobuf), unit, TRUE);
+	sy += closeup_spacing;
 	MoveTo(sx, sy);
-	DrawText(tmpbuf, 0, strlen(tmpbuf));
-	/* Draw the unit's current ACP, if applicable. */
-	if (u_acp(u) > 0) {
-		acp_desc(tmpbuf, unit, TRUE);
-		sy += closeupspacing;
-		MoveTo(sx, sy);
-		DrawText(tmpbuf, 0, strlen(tmpbuf));
-	}
+	DrawText(infobuf, 0, strlen(infobuf));
 	/* Draw the unit's supplies. */
-	count = 0;
-	if (nummrows[u] > 0)
-	  sy += closeupspacing;
-	for_all_material_types(m) {
-		if (um_storage_x(u, m) > 0) {
-			strcpy(tmpbuf, m_type_name(m));
-			sx2 = sx + (count % m_per_row) * (closeupwinwid / m_per_row);
-			sy2 = sy + (count / m_per_row) * closeupspacing;
-			MoveTo(sx2, sy2);
-			DrawText(tmpbuf, 0, strlen(tmpbuf));
-			sprintf(tmpbuf, "%d/%d", unit->supply[m], um_storage_x(u, m));
-			MoveTo(sx2 + (closeupwinwid / m_per_row - 4 - 4 - TextWidth(tmpbuf, 0, strlen(tmpbuf))), sy2);
-			DrawText(tmpbuf, 0, strlen(tmpbuf));
-			++count;
-		}
+	mrow = 0;
+	while (supply_desc(infobuf, unit, mrow)) {
+		sy += closeup_spacing;
+		MoveTo(sx, sy);
+		DrawText(infobuf, 0, strlen(infobuf));
+		++mrow;
 	}
-	if (nummrows[u] > 0)
-	  sy += (nummrows[u] - 1) * closeupspacing; 
 	/* Draw the unit's plan, if it has one. */
 	if (unit->plan) {
 		Task *task;
-		extern char *plantypenames[];
 		Plan *plan = unit->plan;
 		
-		sprintf(tmpbuf, "Plan: %s", plantypenames[plan->type]);
-    	if (plan->waitingfortasks)
-    	  strcat(tmpbuf, " [wait]");
-    	if (plan->delayed) 
-    	  strcat(tmpbuf, " [delay]");
-    	if (plan->reserve)
-    	  strcat(tmpbuf, " [resv]");
-    	if (plan->asleep)
-    	  strcat(tmpbuf, " [aslp]");
-    	if (plan->aicontrol)
-    	  strcat(tmpbuf, " [dlgt]");
-    	if (plan->supply_is_low)
-    	  strcat(tmpbuf, " [low]");
-    	if (plan->supply_alarm)
-    	  strcat(tmpbuf, " [alrm]");
-		sy += closeupspacing;
+		plan_desc(infobuf, unit);
+		sy += closeup_spacing;
 		MoveTo(sx, sy);
-		DrawText(tmpbuf, 0, strlen(tmpbuf));
+		DrawText(infobuf, 0, strlen(infobuf));
     	if (plan->maingoal) {
-    		/* (should use a "goal_desc" routine) */
-    		strcpy(tmpbuf, goal_desig(plan->maingoal));
-			sy += closeupspacing;			
+			strcpy(infobuf, "Goal: ");
+    		goal_desc(infobuf+strlen(infobuf), plan->maingoal);
+			sy += closeup_spacing;			
 			MoveTo(sx, sy);
-			DrawText(tmpbuf, 0, strlen(tmpbuf));
+			DrawText(infobuf, 0, strlen(infobuf));
     	}
     	if (plan->formation) {
-    		/* (should use a "goal_desc" routine) */
-    		strcpy(tmpbuf, goal_desig(plan->formation));
-			sy += closeupspacing;			
+			strcpy(infobuf, "Formation: ");
+    		goal_desc(infobuf+strlen(infobuf), plan->formation);
+			sy += closeup_spacing;			
 			MoveTo(sx, sy);
-			DrawText(tmpbuf, 0, strlen(tmpbuf));
+			DrawText(infobuf, 0, strlen(infobuf));
     	}
     	for_all_tasks(plan, task) {
-			task_desc(tmpbuf, task);
-			sy += closeupspacing;			
+			task_desc(infobuf, unit->side, task);
+			sy += closeup_spacing;			
 			MoveTo(sx, sy);
-			DrawText(tmpbuf, 0, strlen(tmpbuf));
+			DrawText(infobuf, 0, strlen(infobuf));
 		}
 	}
 	SetPort(oldport);
@@ -1518,14 +1511,15 @@ destroy_unit_closeup(UnitCloseup *unitcloseup)
 
 /* History window. */
 
-int maxvishistlines = 200;
-
 void
 create_history_window()
 {
 	Rect vscrollrect;
 
 	historywin = GetNewWindow(wHistory, NULL, (WindowPtr) -1L);
+	SetPort(historywin);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	/* (should calc max based on size of font and height of screen) */
 	histcontents = (HistEvent **) xmalloc(maxvishistlines * sizeof(HistEvent *));
 	vscrollrect = historywin->portRect;
@@ -1535,90 +1529,55 @@ create_history_window()
 	vscrollrect.right += 1;
 	histvscrollbar = NewControl(historywin, &vscrollrect, "\p", TRUE,
 			 					0, 0, 100, scrollBarProc, 0L);
-	firstvisevt = history->next;
-	secondvisevt = firstvisevt->next;
-	update_total_hist_lines();
+	total_history_lines = update_total_hist_lines(dside);
 	set_history_scrollbar();
 	ShowWindow(historywin);
 }
 
-void
+static void
 calc_history_layout()
 {
-	update_total_hist_lines();
+	total_history_lines = update_total_hist_lines(dside);
 	set_history_scrollbar();
+	numhistcontents = build_hist_contents(dside, GetCtlValue(histvscrollbar), histcontents,
+										  numvishistlines);
 }
 
-void
-update_total_hist_lines()
-{
-	HistEvent *hevt;
-	
-	total_history_lines = 0;
-	for (hevt = history->next; hevt != history; hevt = hevt->next) {
-		if (side_in_set(dside, hevt->observers)) {
-			if (hevt->startdate != hevt->prev->startdate) ++total_history_lines;
-			++total_history_lines;
-		}
-	}
-}
-
-void
+static void
 set_history_scrollbar()
 {
-	int hgt, oldmax;
+	int hgt, oldval, oldmax;
 	HistEvent *nexthevt;
 
+	/* Compute the number of lines available for displaying history. */
 	hgt = historywin->portRect.bottom - historywin->portRect.top;
-	numvishistlines = (hgt - history_line_spacing - sbarwid) / history_line_spacing;
+	history_line_spacing = small_line_spacing;
+	history_top_line_height = 2 * small_line_spacing;
+	numvishistlines = (hgt - history_top_line_height - sbarwid) / history_line_spacing;
+	/* Tweak the scrollbar appropriately. */
+	oldval = GetCtlValue(histvscrollbar);
 	oldmax = GetCtlMax(histvscrollbar);
 	SetCtlMax(histvscrollbar, max(0, total_history_lines - numvishistlines + 1));
-	HiliteControl(histvscrollbar, (numvishistlines < total_history_lines ? 0 : 255));
-	/* If the thumb was at max, move it to the new max. */
-	if (GetCtlValue(histvscrollbar) == oldmax) {
-		SetCtlValue(histvscrollbar, GetCtlMax(histvscrollbar));
-		if (GetCtlValue(histvscrollbar) == 0) {
-			firstvisevt = history->next;
-			secondvisevt = firstvisevt->next;
-		} else {
-			firstvisevt = get_nth_history_line(dside, GetCtlValue(histvscrollbar), &nexthevt);
-			secondvisevt = nexthevt;
-		}
-	}
+	HiliteControl(histvscrollbar, (GetCtlMax(histvscrollbar) > 0 ? 0 : 255));
+	/* If the thumb was present and at max, move it to the new max. */
+	if (oldmax > 0 && oldval == oldmax)
+	  SetCtlValue(histvscrollbar, GetCtlMax(histvscrollbar));
 }
 
 void
 draw_history()
 {
-	int i, headdate;
+	int i, n, headdate;
     HistEvent *hevt;
 	int numchars;
 	char *datestr, hdatebuf[100];
 
 	if (!active_display(dside) || historywin == nil)
 	  return;
-	/* Build up the array of events and dates to draw. */
-	numhistcontents = 0;
-	hevt = firstvisevt;
-	if (hevt == NULL) {
-		histcontents[numhistcontents++] = NULL;
-		hevt = secondvisevt;
-	}
-	for (; hevt != history; hevt = hevt->next) {
-		if (numhistcontents >= numvishistlines)
-		  break;
-		if (side_in_set(dside, hevt->observers)) {
-			if (numhistcontents > 0
-			    && histcontents[numhistcontents - 1] != NULL
-			    && hevt->startdate != histcontents[numhistcontents - 1]->startdate) {
-				histcontents[numhistcontents++] = NULL;
-			}
-			histcontents[numhistcontents++] = hevt;
-		}
-	}
+	numhistcontents = build_hist_contents(dside, GetCtlValue(histvscrollbar), histcontents, numvishistlines);
 	/* Draw the header line. */
-	MoveTo(2, 10);
-	headdate = (firstvisevt ? firstvisevt : secondvisevt)->startdate; 
+	MoveTo(2, small_line_spacing);
+	headdate = (histcontents[0] ? histcontents[0] : histcontents[1])->startdate; 
 	/* (should be relative) */
 	datestr = absolute_date_string(headdate);
 	sprintf(hdatebuf, "(%s)", datestr);
@@ -1628,48 +1587,38 @@ draw_history()
 	/* Now draw each event or date. */
 	for (i = 0; i < numhistcontents; ++i) {
 		if (histcontents[i] != NULL) {
-			draw_historical_event(histcontents[i], i);
+			draw_historical(histcontents[i], i, TRUE);
 		} else {
-			draw_historical_date(histcontents[i+1], i);
+			draw_historical(histcontents[i+1], i, FALSE);
 		}
 	}
 }
 
 void
-draw_historical_event(HistEvent *hevt, int y)
+draw_historical(HistEvent *hevt, int y, int drawevt)
 {
 	int hgt, pos, numchars;
-	char hevtbuf[500];
+	char *datestr, buf[500];
 
 	if (hevt == NULL)
 	  return;
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	pos = history_line_spacing * y + history_top_line_height;
 	hgt = historywin->portRect.bottom - historywin->portRect.top;
 	/* Don't draw the line if it will intrude on the bottom scrollbar. */
 	if (pos + history_line_spacing > hgt - sbarwid)
 	  return;
-	MoveTo(20, pos);
-	historical_event_desc(dside, hevt, hevtbuf);
+	MoveTo((drawevt ? 20 : 2), pos);
+	if (drawevt)
+	  historical_event_desc(dside, hevt, buf);
+	else {
+		datestr = absolute_date_string(hevt->startdate);
+		strcpy(buf, datestr);
+	}
 	/* (should clip to drawing only visible chars) */
-	numchars = strlen(hevtbuf);
-	DrawText(hevtbuf, 0, numchars);
-}
-
-void
-draw_historical_date(HistEvent *hevt, int y)
-{
-	int numchars;
-	char *datestr, hdatebuf[100];
-
-	if (hevt == NULL)
-	  return;
-	MoveTo(2, history_line_spacing * y + history_top_line_height);
-	/* (should be relative) */
-	datestr = absolute_date_string(hevt->startdate);
-	strcpy(hdatebuf, datestr);
-	/* (should clip to drawing only visible chars) */
-	numchars = strlen(hdatebuf);
-	DrawText(hdatebuf, 0, numchars);
+	numchars = strlen(buf);
+	DrawText(buf, 0, numchars);
 }
 
 void
@@ -1677,12 +1626,14 @@ update_history_window(HistEvent *hevt)
 {
 	HistEvent *prevfirst, *prevsecond;
 
-	prevfirst = firstvisevt;
-	prevsecond = secondvisevt;
+	prevfirst = histcontents[0];
+	prevsecond = histcontents[1];
 	SetPort(historywin);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	calc_history_layout();
-	if (firstvisevt != prevfirst
-	    || secondvisevt != prevsecond
+	if (histcontents[0] != prevfirst
+	    || histcontents[1] != prevsecond
 	    || numvishistlines > total_history_lines) {
 		force_update(historywin);
 	}
@@ -1720,17 +1671,16 @@ history_scroll_fn(ControlHandle control, short code)
 void
 do_mouse_down_history(Point mouse, int mods)
 {
-	HistEvent *prevfirst, *prevsecond, *nexthevt;
 	ControlHandle control;
-	short part, value;
+	short oldval, part, value;
 
 	if (history_scroll_proc == NULL)
 	  history_scroll_proc = NewControlActionProc(history_scroll_fn);
 
+	oldval = GetCtlValue(histvscrollbar);
+
 	part = FindControl(mouse, historywin, &control);
 	if (control == histvscrollbar) {
-		prevfirst = firstvisevt;
-		prevsecond = secondvisevt;
 		switch (part) {
 			case inThumb:
 				part = TrackControl(control, mouse, NULL);
@@ -1740,9 +1690,9 @@ do_mouse_down_history(Point mouse, int mods)
 				break;
 		}
 		value = GetCtlValue(control);
-		firstvisevt = get_nth_history_line(dside, value, &nexthevt);
-		secondvisevt = nexthevt;
-		if (firstvisevt != prevfirst || secondvisevt != prevsecond) {
+		if (value != oldval) {
+			Dprintf("New scroll is %d, old was %d\n", value, oldval);
+			numhistcontents = build_hist_contents(dside, value, histcontents, numvishistlines);
 			force_update(historywin);
 		}
 	} else {
@@ -1789,7 +1739,7 @@ zoom_history(int part)
 		InsetRect(&zoomrect, 4, 4);
 		/* If not much history, shrink the zoomed window to fit. */
 		vislinesavail = (zoomrect.bottom - zoomrect.top - sbarwid) / history_line_spacing;
-		update_total_hist_lines();
+		total_history_lines = update_total_hist_lines(dside);
 		if (vislinesavail > total_history_lines) {
 			zoomrect.bottom = zoomrect.top + total_history_lines * history_line_spacing + sbarwid;
 		}
@@ -1837,11 +1787,8 @@ create_notice_window()
 		noticewin = GetNewWindow(wNotice, NULL, (WindowPtr) -1L);
 	}
 	SetPort(noticewin);
-	/* All text will be in Times. */
-	/* (should these be choosable?) */
-	TextFont(times);
-	/* Set up the notice text. */
-	TextSize(14);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	h = window_width(noticewin);  v = window_height(noticewin);
 	SetRect(&viewrect, 5, 5, h - sbarwid, v - sbarwid); 
 	destrect = viewrect;
@@ -1903,9 +1850,10 @@ draw_notice()
 
 	GetPort(&oldport);
 	SetPort(noticewin);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	SetRect(&tmprect, 5, 40, 5 + 32, 40 + 32);
 	EraseRect(&tmprect);
-	TextSize(14);
 	TEUpdate(&(noticewin->portRect), notice_text);
 	SetPort(oldport);
 	adjust_notice_scrollbar();
@@ -2114,11 +2062,8 @@ create_scores_window()
 		scoreswin = GetNewWindow(wScores, NULL, (WindowPtr) -1L);
 	}
 	SetPort(scoreswin);
-	/* All text will be in Times. */
-	/* (should these be choosable?) */
-	TextFont(times);
-	/* Set up the scores text. */
-	TextSize(14);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	h = window_width(scoreswin);  v = window_height(scoreswin);
 	SetRect(&viewrect, 5, 5, h - sbarwid, v - sbarwid); 
 	destrect = viewrect;
@@ -2132,7 +2077,7 @@ create_scores_window()
 	scores_v_scrollbar =
 		NewControl(scoreswin, &vscrollrect, "\p", TRUE, 0, 0, 0, scrollBarProc, 0L);
 	HiliteControl(scores_v_scrollbar, 0);
-	add_window_menu_item("scores", scoreswin);
+	add_window_menu_item("Scores", scoreswin);
 }
 
 void
@@ -2172,9 +2117,10 @@ draw_scores()
 
 	GetPort(&oldport);
 	SetPort(scoreswin);
+	TextFont(small_font_id);
+	TextSize(small_font_size);
 	SetRect(&tmprect, 5, 40, 5 + 32, 40 + 32);
 	EraseRect(&tmprect);
-	TextSize(14);
 	TEUpdate(&(scoreswin->portRect), scores_text);
 	SetPort(oldport);
 	adjust_scores_scrollbar();
