@@ -8,6 +8,7 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "xconq.h"
+extern void init_emblem PARAMS ((Side *side, Side *side2));
 
 extern XrmOptionDescRec xoptions[];
 extern int xoptions_count;
@@ -25,18 +26,12 @@ static void init_cursors PARAMS ((Side *side));
 static void init_unit_images PARAMS ((Side *side));
 static void init_terrain_images PARAMS ((Side *side));
 static void init_emblem_images PARAMS ((Side *side));
-static void init_emblem PARAMS ((Side *side, Side *side2));
-static void get_default_terrain_image PARAMS ((Side *side, ImageFamily *imf,
-					      int t));
 static void get_default_emblem PARAMS ((Side *side, ImageFamily *imf,
 				       Side *side2));
 static void report_missing_images PARAMS ((Side *side));
 static void describe_imf PARAMS ((Side *side, char *classname, char *typename,
 				 ImageFamily *imf));
 
-static void x11_interp_fn PARAMS ((ImageFamily *imf));
-static void x11_load_fn PARAMS ((ImageFamily *imf));
-
 typedef struct s_varw {
   Widget dialog;
   char *value;
@@ -123,9 +118,11 @@ char *imflib = IMFLIB;
 /* Display and window globals used in callbacks from generic imf
    handling code. */
 
-Display *tmp_display;
+extern int tmp_valid;
 
-Window tmp_root_window;
+extern Display *tmp_display;
+
+extern Window tmp_root_window;
 
 /* How much space to leave for a unit image, if all images should get
    the same amount (such as for a list of unit types). */
@@ -366,8 +363,6 @@ Side *side;
 	    side->ui->unitchars[p][u] = '\0';
 	    side->ui->unitfonts[p][u] = NULL;
 	    side->ui->ulegendfonts[p][u] = NULL;
-	    /* (these values really should be filled in better) */
-	    side->ui->unitw[p][u] = 1;  side->ui->unith[p][u] = 1;
 	}
     }
     for (s = 0; s < MAXSIDES; ++s) {
@@ -376,9 +371,15 @@ Side *side;
     }
     set_colors(side);
     init_fonts(side);
+    imf_interp_hook = x11_interp_imf;
+    imf_load_hook = x11_load_imf;
+    tmp_display = side->ui->dpy;
+    tmp_root_window = side->ui->rootwin;
+    tmp_valid = TRUE;
     init_unit_images(side);
     init_terrain_images(side);
     init_emblem_images(side);
+    tmp_valid = FALSE;
     report_missing_images(side);
     set_optional_colors(side);
     init_bitmaps(side);
@@ -824,20 +825,6 @@ XFontStruct *font;
     return font->max_bounds.ascent + font->max_bounds.descent;
 }
 
-static void
-x11_interp_fn(imf)
-ImageFamily *imf;
-{
-    x11_interp_imf(tmp_display, tmp_root_window, imf, FALSE);
-}
-
-static void
-x11_load_fn(imf)
-ImageFamily *imf;
-{
-    x11_load_imf(tmp_display, tmp_root_window, imf);
-}
-
 static void
 init_unit_images(side)
 Side *side;
@@ -851,8 +838,7 @@ Side *side;
     tmp_display = side->ui->dpy;
     tmp_root_window = side->ui->rootwin;
     for_all_unit_types(u) {
-	imf = get_unit_type_images(side, u, "s", x11_interp_fn, x11_load_fn,
-				   NULL);
+	imf = get_unit_type_images(side, u, "c");
 	/* (should fabricate something as a substitute) */
 	if (DebugG)
 	  describe_imf(side, "unit type", u_type_name(u), imf);
@@ -880,13 +866,8 @@ Side *side;
 
     side->ui->timages =
       (ImageFamily **) xmalloc(numttypes * sizeof(ImageFamily *));
-    tmp_display = side->ui->dpy;
-    tmp_root_window = side->ui->rootwin;
     for_all_terrain_types(t) {
-	imf = get_terrain_type_images(side, t, x11_interp_fn, x11_load_fn,
-				      NULL);
-	if (imf != NULL && imf->numsizes == 0)
-	  get_default_terrain_image(side, imf, t);
+	imf = get_terrain_type_images(side, t);
 	if (DebugG)
 	  describe_imf(side, "terrain type", t_type_name(t), imf);
 	side->ui->timages[t] = imf;
@@ -909,7 +890,7 @@ Side *side;
 	}
 	/* Look for a solid color. */
 	timg = best_image(side->ui->timages[t], 1, 1);
-	if (timg->w == 1 && timg->h == 1) {
+	if (timg != NULL && timg->w == 1 && timg->h == 1) {
 	    if (timg->hook) {
 		ximg = (X11Image *) timg->hook;
 		/* Save the color if found. */
@@ -920,37 +901,6 @@ Side *side;
     }
 }
 
-/* The default terrain image is ugly but functional; basically a binary
-   encoding of the terrain type number, enclosed in a box. */
-
-static void
-get_default_terrain_image(side, imf, t)
-Side *side;
-ImageFamily *imf;
-int t;
-{
-    char bits[8];
-    Image *img;
-    X11Image *ximg;
-
-    img = get_img(imf, 8, 8);
-    img->minw = 1;  img->minh = 1;
-    img->maxw = 999;  img->maxh = 999;
-    img->istile = TRUE;
-    ximg = get_x11_image(img);
-    bits[0] = 0x01;
-    bits[1] = (t << 2) + 0x01;
-    bits[2] = (t << 2) + 0x01;
-    bits[3] = 0x7d;
-    bits[4] = (t << 2) + 0x01;
-    bits[5] = (t << 2) + 0x01;
-    bits[6] = 0x01;
-    bits[7] = 0xff;
-    ximg->mono = 
-      XCreateBitmapFromData(side->ui->dpy, side->ui->rootwin, bits, 8, 8);
-    imf->ersatz = TRUE;
-}
-
 /* Set up a side's view of everybody else's colors and emblems. */
 
 static void
@@ -960,7 +910,9 @@ Side *side;
     Side *side2;
     
     side->ui->eimages =
-      (ImageFamily **) xmalloc((numsides + 1) * sizeof(ImageFamily *));
+      (ImageFamily **) xmalloc((g_sides_max() + 1) * sizeof(ImageFamily *));
+    side->ui->eimages_loaded =
+      (short *) xmalloc((g_sides_max() + 1) * sizeof(short));
     /* Independent units may have a distinguishing emblem, so the
        indepside is included here. */
     for_all_sides_plus_indep(side2) {
@@ -974,7 +926,7 @@ Side *side;
    and colors that the sides with displays will see, but that sides
    without displays don't need to do any emblem init. */
 
-static void
+void
 init_emblem(side, side2)
 Side *side, *side2;
 {
@@ -1004,15 +956,18 @@ Side *side, *side2;
     }
     tmp_display = side->ui->dpy;
     tmp_root_window = side->ui->rootwin;
-    imf = get_emblem_images(side, side2, x11_interp_fn, x11_load_fn, NULL);
+    tmp_valid = TRUE;
+    imf = get_emblem_images(side, side2);
     if (imf != NULL
 	&& imf->numsizes == 0
 	&& !(imf->name != NULL && strcmp(imf->name, "none") == 0)) {
 	get_default_emblem(side, imf, side2);
     }
+    tmp_valid = FALSE;
     if (DebugG)
       describe_imf(side, "emblem", side_desig(side2), imf);
     side->ui->eimages[s2] = imf;
+    side->ui->eimages_loaded[s2] = TRUE;
 }
 
 static void
@@ -1021,12 +976,8 @@ Side *side, *side2;
 ImageFamily *imf;
 {
     Image *img;
-    X11Image *ximg;
 
     img = get_img(imf, 8, 8);
-    img->minw = 8;  img->minh = 8;
-    img->maxw = 128;  img->maxh = 128;
-    ximg = get_x11_image(img);
     /* (should make something for this image) */
 }
 
@@ -1051,7 +1002,7 @@ ImageFamily *imf;
     if (imf->location)
       DGprintf(" and is in %s", imf->location->name);
     DGprintf("\n");
-    for (img = imf->images; img != NULL; img = img->next) {
+    for_all_images(imf, img) {
 	DGprintf("    %dx%d", img->w, img->h);
 	ximg = (X11Image *) img->hook;
 	if (ximg)
